#========================================================================================
# package      DE_EPAGES::Amazon::API::SOM::OrderReportParser
# state        public
# description  parser for OrderReport documents.
#========================================================================================
package DE_EPAGES::Amazon::API::SOM::OrderReportParser;

use strict;

use DE_EPAGES::Amazon::API::DTO::OrderReport;
use DE_EPAGES::Amazon::API::DTO::Item;
use DE_EPAGES::Amazon::API::DTO::Address;
use DE_EPAGES::Amazon::API::DTO::Component;
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Amazon::API::Tools qw ( WantList );

#========================================================================================
# function     Parse
# state        public
#----------------------------------------------------------------------------------------
# syntax       Parse( $SOMInput, $documentIdentifier );
#----------------------------------------------------------------------------------------
# description  handler for SOM structure of order report document.
#----------------------------------------------------------------------------------------
# input        $SOMInput | SOM tree | object
# input        $documentIdentifier | document identifiert | string
# return       $aReports | report list | ref.array.DE_EPAGES::Amazon::API::DTO::OrderReport
#========================================================================================
sub Parse {    ## no critic (DE_EPAGES::Style::ProhibitFunctionSize)
    my ($som, $documentIdentifier) = @_;

    my @ReportList;

    for ( my $i=1; $som->match('/AmazonEnvelope/['.$i.']'); ++$i ) { ## no critic (ProhibitFunctionSize)
        my $Message = $som->valueof('/AmazonEnvelope/['.$i.']/OrderReport') or next;
        # find used currency.
        my $_Amount = $som->dataof('/AmazonEnvelope/['.$i.']/OrderReport/Item/ItemFees/Fee/Amount');
        if (! $_Amount) {
            $_Amount = $som->dataof('/AmazonEnvelope/['.$i.']/OrderReport/Item/ItemPrice/[1]/Amount');
            #no fee in xml
            Warning( 'AmazonOrder_NoFeeFound' , {'AmazonOrderID' => $Message->{AmazonOrderID} });
        }
        my $Currency = defined $_Amount ? $_Amount->attr->{'currency'} : undef;
        if (! $Currency) {
            #no currency in first item in xml
            Warning( 'AmazonOrder_NoCurrencyFound', {'AmazonOrderID' => $Message->{AmazonOrderID} } );
        }

        my $Report = DE_EPAGES::Amazon::API::DTO::OrderReport->new;
        $Report->ReportNumber( $documentIdentifier );
        $Report->AmazonOrderID( $Message->{AmazonOrderID} );
        $Report->AmazonSessionID( $Message->{AmazonSessionID} );
        $Report->OrderDate( $Message->{OrderDate} );
        $Report->BuyerEmailAddress( $Message->{BillingData}->{BuyerEmailAddress} );
        $Report->BuyerName( $Message->{BillingData}->{BuyerName} );
        $Report->BuyerPhoneNumber( $Message->{BillingData}->{BuyerPhoneNumber} );

        my $BuyerAddress = DE_EPAGES::Amazon::API::DTO::Address->new;
        $BuyerAddress->Name( $Message->{BillingData}->{Address}->{Name} );
        $BuyerAddress->AddressFieldOne( $Message->{BillingData}->{Address}->{AddressFieldOne} );
        $BuyerAddress->AddressFieldTwo( $Message->{BillingData}->{Address}->{AddressFieldTwo} );
        $BuyerAddress->City( $Message->{BillingData}->{Address}->{City} );
        $BuyerAddress->PostalCode( $Message->{BillingData}->{Address}->{PostalCode} );
        $BuyerAddress->CountryCode( $Message->{BillingData}->{Address}->{CountryCode} );
        $BuyerAddress->StateOrRegion( $Message->{BillingData}->{Address}->{StateOrRegion} );
        $BuyerAddress->PhoneNumber( $Message->{BillingData}->{Address}->{PhoneNumber} );
        $Report->BuyerAddress($BuyerAddress);

        if ( defined $Message->{FulfillmentData}->{Address} ) {
            my $FulfillmentAddress = DE_EPAGES::Amazon::API::DTO::Address->new;
            $FulfillmentAddress->Name( $Message->{FulfillmentData}->{Address}->{Name} );
            $FulfillmentAddress->AddressFieldOne( $Message->{FulfillmentData}->{Address}->{AddressFieldOne} );
            $FulfillmentAddress->AddressFieldTwo( $Message->{FulfillmentData}->{Address}->{AddressFieldTwo} );
            $FulfillmentAddress->City( $Message->{FulfillmentData}->{Address}->{City} );
            $FulfillmentAddress->PostalCode( $Message->{FulfillmentData}->{Address}->{PostalCode} );
            $FulfillmentAddress->CountryCode( $Message->{FulfillmentData}->{Address}->{CountryCode} );
            $FulfillmentAddress->StateOrRegion( $Message->{FulfillmentData}->{Address}->{StateOrRegion} );
            $FulfillmentAddress->PhoneNumber( $Message->{FulfillmentData}->{Address}->{PhoneNumber} );
            $Report->FulfillmentAddress($FulfillmentAddress);
        }

        $Report->FulfillmentMethod( $Message->{FulfillmentData}{FulfillmentMethod} );
        $Report->FulfillmentServiceLevel( $Message->{FulfillmentData}->{FulfillmentServiceLevel} );

        foreach my $MItem ( WantList( $Message->{Item} ) ) {

            my $Item = DE_EPAGES::Amazon::API::DTO::Item->new;
            $Item->AmazonOrderItemCode( $MItem->{AmazonOrderItemCode} );
            $Item->GiftMessageText( $MItem->{GiftMessageText} );
            $Item->GiftWrapLevel( $MItem->{GiftWrapLevel} );
            $Item->SKU( $MItem->{SKU} );
            $Item->Title( $MItem->{Title} );
            $Item->Quantity( $MItem->{Quantity} );
            $Item->ProductTaxCode( $MItem->{ProductTaxCode} );

            if ( $MItem->{ItemPrice} ) {
                my $ItemPrice = $MItem->{ItemPrice};
                foreach my $MPrice ( WantList( $ItemPrice->{Component} ) ) {
                    my $Price = DE_EPAGES::Amazon::API::DTO::Component->new;
                    $Price->Amount( $MPrice->{Amount} );
                    $Report->Currency( $Currency );
                    $Price->Type( $MPrice->{Type} );
                    $Item->ItemPriceArray($Price);
                }
            }

            if ( $MItem->{Promotion} ) {
                foreach my $ItemPromotion ( WantList( $MItem->{Promotion} )) {
                    foreach my $MPrice ( WantList($ItemPromotion->{Component}) ) {
                        my $Price = DE_EPAGES::Amazon::API::DTO::Component->new;
                        my $Promo = $ItemPromotion->{MerchantPromotionID};
                        $Price->ItemPromotionCode($Promo);
                        $Price->IsPromotion(1);
                        $Price->Amount( $MPrice->{Amount} );
                        $Price->Type( $MPrice->{Type} );
                        $Item->ItemPriceArray($Price);
                    }
                }
            }

            $Report->ItemArray($Item);
        }
        push @ReportList, $Report;
    }
    return \@ReportList;
}

1;