#========================================================================================
# package      DE_EPAGES::CrossSelling::Hooks::Product
# state        private
# description  removes cross-selling references when the target product is deleted.
#========================================================================================
package DE_EPAGES::CrossSelling::Hooks::Product;

use strict;
use DE_EPAGES::CrossSelling::API::Constants qw ( UNITY_CROSS_SELLING_NOTIFICATION_THRESHOLD );
use DE_EPAGES::CrossSelling::API::Table::AutoCrossSelling qw (DeleteAutoCrossSelling
                                                              GetPKeyAutoCrossSellingsByProductID1
                                                              GetPKeyAutoCrossSellingsByProductID2);
use DE_EPAGES::Object::API::Factory qw( LoadClassByAlias );


#========================================================================================
# function     OnInsertProduct
# state        private
#----------------------------------------------------------------------------------------
# syntax       OnInsertProduct( $hParams );
#----------------------------------------------------------------------------------------
# description  Inserts the cross-selling target product from the CrossSelling table.
#               Inserts auto-cross-selling entries.
#               (Hook OBJ_InsertProduct)
#----------------------------------------------------------------------------------------
# input        $hParams | hash parameters, key is
#               <ul>
#                   <li>Object - Product object - object
#               </ul> | ref.hash.*
#========================================================================================
sub OnInsertProduct {
    my ($hParams) = @_;
    my $Product = $hParams->{'Object'};
    my $Shop = $Product->getSite();

    # we do the following actions only for unity shops that adds the 15th product
    # and has no cross selling set up so far
    if ($Shop->get('IsUnityRegistered') &&
        $Shop->featureCurrentValue('Products') == UNITY_CROSS_SELLING_NOTIFICATION_THRESHOLD &&
        $Shop->featureMaxValue('ManualCrossSelling')) {

        my $CrossSellingClass = LoadClassByAlias('CrossSelling');

        if(scalar @{$CrossSellingClass->findObjects('BySiteIDClassID', [$Shop->id, $CrossSellingClass->id])} == 0) {
            my $CrossSellingNotification
                = $Shop->folder('ShopNotifications')->child('CrossSelling');
            return unless defined $CrossSellingNotification;

            $CrossSellingNotification->addObject($Shop);
        }
    }

    return;
}

#========================================================================================
# function     OnDeleteProduct
# state        private
#----------------------------------------------------------------------------------------
# syntax       OnDeleteProduct( $hParams );
#----------------------------------------------------------------------------------------
# description  Deletes the cross selling target product from the CrossSelling table.
#               Deletes auto-cross selling entries.
#               (Hook OBJ_DeleteProduct)
#----------------------------------------------------------------------------------------
# input        $hParams | hash parameters, key is
#               <ul>
#                   <li>Object - Product object - object
#               </ul> | ref.hash.*
#========================================================================================
sub OnDeleteProduct {
    my ($hParams) = @_;
    my $TargetProduct = $hParams->{'Object'};

    my $ProductID = $TargetProduct->id;

    # delete manual cross selling,accessories and product comparison
    foreach my $ClassAlias (qw( Accessory CrossSelling ProductComparison )) {
        my $Class = LoadClassByAlias($ClassAlias);
        $Class->deleteObjects('ByTargetProductID', [ $ProductID ]);
        $Class->deleteObjects('ByProductID', [ $ProductID ]);
    }

    # delete auto cross selling in both directions
    DeleteAutoCrossSelling($_) foreach @{GetPKeyAutoCrossSellingsByProductID1( $ProductID )};
    DeleteAutoCrossSelling($_) foreach @{GetPKeyAutoCrossSellingsByProductID2( $ProductID )};

    return;
}

#========================================================================================
# function     OnProductSEOCheckRatings
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnProductSEOCheckRatings( $hParams );#----------------------------------------------------------------------------------------
# description  Extends/changes SEO rating percentage values for products
#               (Hook ProductSEOCheckRatings)
#----------------------------------------------------------------------------------------
# input        $hParams | objects
#               <ul>
#                   <li>Object - object - object
#                   <li>Values - percentage values for SEO check ratings - ref.hash
#               </ul> | ref.hash.*
#========================================================================================
sub OnProductSEOCheckRatings {
    my ($hParams) = @_;

    # overwrite with cross-selling added values
    %{$hParams->{'Ratings'}} = (
        'Title'             => 0.25,
        'Url'               => 0.15,
        'Description'       => 0.2,
        'AdvDescription'    => 0.2,
        'ProductImages'     => 0.1,
        'CrossSelling'      => 0.1,
    );
}

#========================================================================================
# function     OnSEOCheckTests
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnSEOCheckTests( $hParams );
#----------------------------------------------------------------------------------------
# description  Extends/changes values from the SEO check for content objects
#               (Hook SEOCheckTests)
#----------------------------------------------------------------------------------------
# input        $hParams | objects
#               <ul>
#                   <li>Object - object - object
#                   <li>Shop - shop object - object
#                   <li>Values - values and ratings for SEO check - ref.hash
#               </ul> | ref.hash.*
#========================================================================================
sub OnSEOCheckTests {
    my ($hParams) = @_;

    my $Product = $hParams->{'Object'};
    return unless $Product->instanceOf('Product');

    my $Shop = $hParams->{'Shop'};
    my $hValues = $hParams->{'Values'};

    my $HasCrossSellingFeature = $Shop->featureMaxValue('ManualCrossSelling');

    my $NumAssignedCategories = scalar(@{$Product->get('Categories')}) || 0;
    my $LinkSum = $NumAssignedCategories;

    my $NumCrossSellingProducts = 0;
    if($HasCrossSellingFeature) {
        if($Product->existsChild('CrossSelling')) {
            $NumCrossSellingProducts += scalar(@{$Product->folder('CrossSelling')->findChildren});
        }
        if($Product->existsChild('Accessory')) {
            $NumCrossSellingProducts += scalar(@{$Product->folder('Accessory')->findChildren});
        }
        $LinkSum += $NumCrossSellingProducts;
    }

    my $CrossSellingRating = _getCrossSellingRating($LinkSum, $Shop, $HasCrossSellingFeature);
    $hValues->{'CrossSelling'} = {
        'AssignedCategories' => $NumAssignedCategories,
        'Rating' => $CrossSellingRating,
        'RatingClass' => qw ( green yellow orange red )[$CrossSellingRating-1],
    };
    $hValues->{'CrossSelling'}->{'Products'} = $NumCrossSellingProducts if $HasCrossSellingFeature;
}

sub _getCrossSellingRating {
    my ($LinkSum, $Shop, $HasCrossSellingFeature) = @_;
    if($HasCrossSellingFeature) {
        return 4 if $LinkSum  < 2;
        return 3 if $LinkSum  < 4;
        return 2 if $LinkSum == 4;
        return 1 if $LinkSum  > 4;
    } else {
        return 4 if $LinkSum  < 2;
        return 3 if $LinkSum == 2;
        return 2 if $LinkSum == 3;
        return 1 if $LinkSum  < 3;
    }
}

1;
