use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Error qw( GetError ExistsError );
use DE_EPAGES::Core::API::Log qw( LogError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::CrossSelling::API::Cartridge;
use DE_EPAGES::CrossSelling::API::Constants qw ( UNITY_CROSS_SELLING_NOTIFICATION_THRESHOLD );
use DE_EPAGES::Object::API::Factory qw ( LoadObject LoadClassByAlias );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::Shop::API::System qw( GetUnityShopIDs );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );

#========================================================================================
# function         InitializeCrossSellingNotification
# state            public
#----------------------------------------------------------------------------------------
# syntax           InitializeCrossSellingNotification();
#----------------------------------------------------------------------------------------
# description      Adds the notification for cross-selling to all Unity shops
#========================================================================================
sub InitializeCrossSellingNotification {

    print "Reading all Unity shops\n";
    my $aShopIDs = GetUnityShopIDs();

    my $Cartridge = DE_EPAGES::CrossSelling::API::Cartridge->new;
    foreach my $ShopID (@$aShopIDs) {
        eval {
            my $Shop = LoadObject($ShopID);

            # import notification
            ImportObjects($Cartridge->xmlDirectory().'/ShopNotifications.xml', $Shop->pathString)
                unless $Shop->folder('ShopNotifications')->existsChild('CrossSelling');

            # check if the shop has more than 15 products
            if ($Shop->featureCurrentValue('Products') >= UNITY_CROSS_SELLING_NOTIFICATION_THRESHOLD &&
                $Shop->featureMaxValue('ManualCrossSelling')) {

                my $CrossSellingClass = LoadClassByAlias('CrossSelling');
                # check if there are no cross selling products present in the shop
                if(scalar @{$CrossSellingClass->findObjects('BySiteIDClassID',
                    [$Shop->id, $CrossSellingClass->id])} == 0) {

                    # activate notification
                    my $CrossSellingNotification
                        = $Shop->folder('ShopNotifications')->child('CrossSelling');
                    $CrossSellingNotification->addObject($Shop);
                }
            }
        };
        warn('Patch failed in shop ' . $ShopID . " with error \"" . GetError() . "\".")
            if ExistsError();
    }

    eval { SynchronizeCache(); };
    LogError(GetError()) if ExistsError();

    return;
}

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $Help;

    GetOptions(
        'help'        => \$Help,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
    );

    usage() if $Help;

    unless ( defined $StoreName ) {
        print STDERR "Missing parameter -storename.\n\n";
        usage();
    }
    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub { InitializeCrossSellingNotification(); },
    );
    my $time2 = time;
    printf "need %.1f seconds\n", $time2 - $time1;
}

sub usage {
    print <<END_USAGE;
Description:
    Adds the cross-selling notification for pending orders to all Unity shops.

Usage:
    perl $0 [options] [flags] datafiles

options:
    -passwd     database user password                          ()
    -storename  name of store                                   ()
flags:
    -help       show the command line options

Example:
    perl $0 -storename Store
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );
