#========================================================================================
# package      DE_EPAGES::DHL::API::TLE::DHLHandler
# state        public
#----------------------------------------------------------------------------------------
# description  TLE Handler for DHL Tracking Information
#========================================================================================
package DE_EPAGES::DHL::API::TLE::DHLHandler;

use strict;
use DateTime::Format::ISO8601;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::Warning qw ( BlockWarnings Warning );
use DE_EPAGES::DHL::API::Table::DHLProductList qw ( GetInfoDHLProductListsByProductID );
use DE_EPAGES::DHL::API::Tools qw (
    DHLShippingLabelIsDHLDeliveryDestInDB
    GetMaxShippingHeightFromDB
    GetMaxShippingLengthFromDB
    GetMaxShippingWeightFromDB
    GetMaxShippingWidthFromDB
    GetMinShippingHeightFromDB
    GetMinShippingLengthFromDB
    GetMinShippingWeightFromDB
    GetMinShippingWidthFromDB
);
use DE_EPAGES::Object::API::Factory qw ( LoadObject );

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler = new DE_EPAGES::Object::API::TLE::DHLHandler;
# example      my $PrefixHandler  = new DE_EPAGES::Object::API::TLE::DHLHandler;
#----------------------------------------------------------------------------------------
# description  create a new handler
#----------------------------------------------------------------------------------------
# input        %options | options | hash
# return       $Handler | blessed object | object
#========================================================================================
sub new {
    my $class     = shift;
    my (%options) = @_;
    my $self      = bless \%options, $class;

    return $self;
}

#========================================================================================
# function     DHLTrackingEvents
# state        public
#----------------------------------------------------------------------------------------
# syntax       #DHLTrackingEvents(#OrderID)
# example      #LOOP(#DHLTrackingEvents(#OrderID)) .. #ENDLOOP
#----------------------------------------------------------------------------------------
# description  Loops over a list of all DHL shipping labels for one order to find
#               all available tracking events.
#               Available keys are:
#               <li>IssueDate - ShippingLabel.IssueDate
#               <li>LabelId - ShippingLabel.LabelId
#               <li>ShipmentId - ShippingLabel.ShipmentId
#----------------------------------------------------------------------------------------
# input        $Processor | TLE processor | object
# input        $aParams | #OrderID | array
# return       @List | list of tracking events | array
#========================================================================================
sub DHLTrackingEvents {
    my $self = shift;
    my ( $Processor, $aParams ) = @_;

    my $OrderID = $aParams->[0];
    my $Order   = LoadObject($OrderID);
    my @List    = ();

    if ( $Order->existsChild('DHLShippingLabels') ) {
        my $aDHLShippingLabels = $Order->folder('DHLShippingLabels')->children;
        foreach my $DHLShippingLabel (@$aDHLShippingLabels) {
            my $hLabelInfo = $DHLShippingLabel->get([qw( ShipmentId LabelId IssueDate )]);
            if ( isValid ($hLabelInfo->{'ShipmentId'}) ) {
                push @List,
                  {
                    'IssueDate'  => DateTime::Format::ISO8601->new()->parse_datetime( $hLabelInfo->{'IssueDate'} ),
                    'LabelId'    => $hLabelInfo->{'LabelId'},
                    'ShipmentId' => $hLabelInfo->{'ShipmentId'},
                  };
            }
        }
    }
    return \@List;
}

#========================================================================================
# function     DHLIsDeliveryDestInDB
# state        public
#----------------------------------------------------------------------------------------
# syntax       #DHLIsDeliveryDestInDB(#ShippingLabelID)
# example      #IF(NOT #DHLIsDeliveryDestInDB(#DHLShippingLabel.ID)) .. #ENDIF
#----------------------------------------------------------------------------------------
# description  Checks if the delivery destination can be found the database. If not the
#               merchant has to get his shipping label via DHL marketplace
#----------------------------------------------------------------------------------------
#----------------------------------------------------------------------------------------
# input        $Processor | TLE processor | object
# input        $aParams | #ShippingLabelID  | array
# return       $InDB | 1 if delivery destination is in the DB | boolean
#========================================================================================
sub DHLIsDeliveryDestInDB {
    my $self = shift;
    my ( $Processor, $aParams ) = @_;

    my $DHLShippingLabelID = $aParams->[0];
    my $result;

    BlockWarnings(sub {
        eval {
            $result = DHLShippingLabelIsDHLDeliveryDestInDB($DHLShippingLabelID);
        };
        if (ExistsError()) {
            Warning(GetError());
        }
    }, [qw( MISSING_COUNTRY MISSING_ADDRESS )]);

    return $result;
}

#========================================================================================
# function     DHLGetDisplayNameByProductID
# state        public
#----------------------------------------------------------------------------------------
# syntax       #DHLGetDisplayNameByProductID(#ProductId)
# example      #DHLGetDisplayNameByProductID(#ProductId)
#----------------------------------------------------------------------------------------
# description  Get display name for product id
#----------------------------------------------------------------------------------------
# input        $Processor | TLE processor | object
# input        $aParams | #ProductId  | array
# return       #DisplayName | Display name for product id | string
#========================================================================================
sub DHLGetDisplayNameByProductID {
    my $self = shift;
    my ( $Processor, $aParams ) = @_;

    my $DHLProductID      = $aParams->[0];
    my $ahDHLProductLists = GetInfoDHLProductListsByProductID($DHLProductID);
    return unless (scalar @{$ahDHLProductLists});
    my $result            = $ahDHLProductLists->[0]->{'DisplayName'};

    return $result;
}

#========================================================================================
# function     tle
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Value = $Handler->tle( $Processor, $TLEName);
#----------------------------------------------------------------------------------------
# description  Register TLEs:
#               #DHLMinShippingLengthFromDB<br/>
#               #DHLMaxShippingLengthFromDB<br/>
#               #DHLMinShippingWidthFromDB<br/>
#               #DHLMaxShippingWidthFromDB<br/>
#               #DHLMinShippingHeightFromDB<br/>
#               #DHLMaxShippingHeightFromDB<br/>
#               #DHLMinShippingWeightFromDB<br/>
#               #DHLMaxShippingWeightFromDB<br/>
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
# input        $TLEName | tle name | string
# return       $Value | return value | integer
#========================================================================================
sub tle {
    my $self = shift;
    my ( $Processor, $TLEName ) = @_;

    my %tleHandlers = (
        'DHLMinShippingLengthFromDB' => \&GetMinShippingLengthFromDB,
        'DHLMaxShippingLengthFromDB' => \&GetMaxShippingLengthFromDB,
        'DHLMinShippingWidthFromDB'  => \&GetMinShippingWidthFromDB,
        'DHLMaxShippingWidthFromDB'  => \&GetMaxShippingWidthFromDB,
        'DHLMinShippingHeightFromDB' => \&GetMinShippingHeightFromDB,
        'DHLMaxShippingHeightFromDB' => \&GetMaxShippingHeightFromDB,
        'DHLMinShippingWeightFromDB' => \&GetMinShippingWeightFromDB,
        'DHLMaxShippingWeightFromDB' => \&GetMaxShippingWeightFromDB,
    );

    if ( defined( $tleHandlers{$TLEName} ) ) {
        return &{ $tleHandlers{$TLEName} };
    }
    return undef;
}

#========================================================================================
# function     existsTLE
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Exists = $Handler->existsTLE($Processor, $TLEName);
#----------------------------------------------------------------------------------------
# description  True if tle name is now, means the TLE #Now is in each context avaiable.
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
# input        $TLEName | tle name (Undef) | string
# return       $exists | 1 if tle exists | boolean
#========================================================================================
sub existsTLE {
    my $self = shift;
    my ( $Processor, $TLEName ) = @_;
    GetLog->debug("DHLHandler.existsTLE($TLEName)");

    return ( defined $self->tle( $Processor, $TLEName ) ) ? 1 : 0;
}

#========================================================================================
# function     RegisterHandlerProc
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RegisterHandlerProc( $hParams );
#----------------------------------------------------------------------------------------
# description  this function registers this attribute handle on the processor via hook
#               function on hook TLEProcessorRegistration
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments keys :
#               <ul>
#                   <li>Processor - tle processor - object
#               </ul> | ref.hash
#========================================================================================
sub RegisterHandlerProc {
    my ($Params) = @_;
    __PACKAGE__->new()->register( $Params->{'Processor'} );
    return;
}

#========================================================================================
# function     register
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->register( $Processor );
#----------------------------------------------------------------------------------------
# description  This function registers this handle on the processor.
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
#========================================================================================
sub register {
    my $self = shift;
    my ($Processor) = @_;
    $Processor->registerHandler( 'FunctionHandler', $self, 'DHLTrackingEvents' );
    $Processor->registerHandler( 'FunctionHandler', $self, 'DHLIsDeliveryDestInDB' );
    $Processor->registerHandler( 'FunctionHandler', $self, 'DHLGetDisplayNameByProductID' );
    $Processor->registerHandler( 'VariableHandler', $self, 'DHLMinShippingLengthFromDB' );
    $Processor->registerHandler( 'VariableHandler', $self, 'DHLMaxShippingLengthFromDB' );
    $Processor->registerHandler( 'VariableHandler', $self, 'DHLMinShippingWidthFromDB' );
    $Processor->registerHandler( 'VariableHandler', $self, 'DHLMaxShippingWidthFromDB' );
    $Processor->registerHandler( 'VariableHandler', $self, 'DHLMinShippingHeightFromDB' );
    $Processor->registerHandler( 'VariableHandler', $self, 'DHLMaxShippingHeightFromDB' );
    $Processor->registerHandler( 'VariableHandler', $self, 'DHLMinShippingWeightFromDB' );
    $Processor->registerHandler( 'VariableHandler', $self, 'DHLMaxShippingWeightFromDB' );
    return;
}

1;
