#========================================================================================
# package      DE_EPAGES::ExternalShipping::API::Object::ShippingLabel
# state        public
# base         DE_EPAGES::Order::API::Object::OrderDocument
# description  object interface for ShippingLabel.
#========================================================================================
package DE_EPAGES::ExternalShipping::API::Object::ShippingLabel;
use base DE_EPAGES::Order::API::Object::OrderDocument;

use strict;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Object::API::Factory qw ( ExistsObject LoadObject LoadObjectByPath );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::Core::API::PerlTools qw (fcmp);
use DE_EPAGES::ExternalShipping::API::Shop qw( GetCurrentShippingOrder );
use DE_EPAGES::ExternalShipping::API::Table::ShippingLabelFFOption qw ( DeleteShippingLabelFFOption GetPKeyShippingLabelFFOptionsByShippingLabelID );
#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $ShippingLabel->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'ShippingLabel' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $ShippingLabel->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShippingLabel = DE_EPAGES::ExternalShipping::API::Object::ShippingLabel->insert($hInfo);
# example      my $Class = LoadClassByAlias('ShippingLabel');
#               my $ShippingLabel = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertShippingLabel' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $ShippingLabel | new object | object
# hook         OBJ_InsertShippingLabel | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShippingLabelID - primary key          - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    my $self = $class->SUPER::insert($hInfo);

    TriggerHook('OBJ_InsertShippingLabel', {'Object' => $self, 'ShippingLabelID' => $self->id });
    delete $hInfo->{$_} foreach qw ( ShippingTransaction ShippingFulfillmentMethod TrackingID TrackingLink ProductCode ShippingFulfillmentProduct LabelStatus TotalWeight WeightUnit Amount CurrencyID CouponCode Insurance ExpressDelivery PickupService PickupDate LabelDownloadURL EstimatedDeliveryDate ActualDeliveryDate ParcelLength ParcelWidth ParcelDepth SizeUnit IsTestLabel ); ## no critic qw(ProhibitLongLines)
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShippingLabel->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteShippingLabel' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteShippingLabel | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShippingLabelID - primary key          - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeleteShippingLabel', {'Object' => $self, 'ShippingLabelID' => $self->id });
    DeleteShippingLabelFFOption($_) foreach @{GetPKeyShippingLabelFFOptionsByShippingLabelID($self->id)};
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShippingLabel->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateShippingLabel' and 'OBJ_AfterUpdateShippingLabel'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateShippingLabel | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShippingLabelID - primary key          - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateShippingLabel | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShippingLabelID - primary key          - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdateShippingLabel', {'Object'=>$self, 'Values'=> $hValues, 'ShippingLabelID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateShippingLabel',  {'Object'=>$self, 'Values'=> $hValues, 'ShippingLabelID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::ExternalShipping::API::Object::ShippingLabel->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;
    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'PickupService' => 0,
        'IsTestLabel' => 0,
        'Order' => $Parent->parent,
    };
}

#========================================================================================
# function     addToShippingOrder
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Label->addToShippingOrder();
#----------------------------------------------------------------------------------------
# description  Adds shipping label to Shipping Order
#========================================================================================
sub addToShippingOrder {
    my $self = shift;
    return if ($self->get({'ShippingTransaction'}));
    #$Shop, $ShippingFulfillmentType
    my $FulfillmentMethod = $self->get('ShippingFulfillmentMethod') or return;
    my $FulfillmentType = $FulfillmentMethod->get('ShippingFulfillmentType');
    my $Transaction = GetCurrentShippingOrder($self->getSite->id, $FulfillmentType);
    $self->set({'ShippingTransaction' => $Transaction});
    return;
}

#========================================================================================
# function     deleteFromShippingOrder
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Label->deleteFromShippingOrder();
#----------------------------------------------------------------------------------------
# description  Deletes shipping label from Shipping Order
#========================================================================================
sub deleteFromShippingOrder {
    my $self = shift;
    $self->set({'ShippingTransaction' => undef });
}

#========================================================================================
# function     issueLabelByHand
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Label->issueLabelByHand();
#----------------------------------------------------------------------------------------
# input        $TrackingID  | Tracking ID | int
#----------------------------------------------------------------------------------------
# description  Sets label to Issued and removes it from ShippingOrder if issued "by hand"
#========================================================================================
sub issueLabelByHand {
    my $self = shift;
    my ($TrackingID) = @_;
    if (! $self->get('Issued')) {
        $self->set({'TrackingID' => $TrackingID });
        $self->deleteFromShippingOrder();
    }
}

#========================================================================================
# function     issueLabelWithTracking
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Label->issueLabelWithTracking($TrackingID);
#----------------------------------------------------------------------------------------
# input        $TrackingID  | Tracking ID | int
#----------------------------------------------------------------------------------------
# description  Sets label to Issued and adds tracking information, if implemented
#               by generateTrackingLink.
#========================================================================================
sub issueLabelWithTracking {
    my $self = shift;
    my ($TrackingID) = @_;
    $self->set({'TrackingID' => $TrackingID });
    return;
}

#========================================================================================
# function     generateTrackingLink
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Label->generateTrackingLink();
#----------------------------------------------------------------------------------------
# description  generates tracking link. Implement in child classes.
#========================================================================================
sub generateTrackingLink { return; }

#========================================================================================
# function     validateLabel
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Label->validateLabel();
#----------------------------------------------------------------------------------------
# description  Does validation for one shipping label.
#               Has to be implemented in child class, e.g. by calling a webservice.
#========================================================================================
sub validateLabel { return; }

#========================================================================================
# function     isValid
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Label->isValid();
#----------------------------------------------------------------------------------------
# description  Returns true if shipping label is completely configured and
#               ready for sending in shipping order. Implementations should save
#               the results of $Label->validateLabel() and return this value here.
#               Non-Webservice implementations may check the existance of all mandatory
#               fields.
#----------------------------------------------------------------------------------------
# return       $IsValid | true if shipping label is valid | boolean
#========================================================================================
sub isValid { return 1; }

#========================================================================================
# function     getCalculatedTotalWeight
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShippingLabel->getCalculatedTotalWeight( );
#----------------------------------------------------------------------------------------
# description  Returns calculated weight based on lineitems in shipping label.
#----------------------------------------------------------------------------------------
# return       $TotalWeight | calculated weight (unit:shippinglabel.weightunit) | float
#========================================================================================
sub getCalculatedTotalWeight {
    my $self = shift;

    my $CalculatedTotalWeight = 0;
    my $aLineItems = $self->get('LineItemContainer')->get('LineItems');
    foreach my $LineItem ( @$aLineItems ) {
        next if not $LineItem->get('Class')->existsAttribute('Weight');
        my $Quantity       = $LineItem->get('Quantity');
        my $Weight         = $LineItem->get('Weight');
        my $WeightUnit     = $LineItem->get('WeightUnit');
        my $WeightQuantity = $LineItem->get('WeightQuantity');
        $WeightQuantity  ||= 1;

        if (defined $Weight and defined $WeightUnit) {
            #--- convert weights to gram and add weights for all line items
            $Weight *= $WeightUnit->get('Factor') if ($WeightUnit->alias ne 'gram');
            $CalculatedTotalWeight += $Weight * $Quantity / $WeightQuantity;
        }
    }
    # convert back to real weightunit
    if ($CalculatedTotalWeight > 0) {
        my $LabelWeightUnit = $self->get('WeightUnit');
        $CalculatedTotalWeight /= $LabelWeightUnit->get('Factor') if ($LabelWeightUnit->alias ne 'gram');
    }

    # Format the total weight value ( float with 3 decimals )
    my $Formatter = DE_EPAGES::Core::API::Object::NumberFormat->new() ;
    $CalculatedTotalWeight = $Formatter->round( $CalculatedTotalWeight, 3 ) ;

    return $CalculatedTotalWeight;
}

#========================================================================================
# function     isTotalWeightCalculated
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShippingLabel->isTotalWeightCalculated();
#----------------------------------------------------------------------------------------
# description  Checks if $ShippingLabel->get('TotalWeight') equals
#               $ShippingLabel->calculatedTotalWeight(). If that's the case total weight
#               can be recalculated when lineitems are changed.
#----------------------------------------------------------------------------------------
# return       $IsEquals | true if total weight is calculated | boolean
#========================================================================================
sub isTotalWeightCalculated {
    my $self = shift;
    my $TotalWeight = $self->get('TotalWeight');
    return 1 unless ($TotalWeight);

    my $CalculatedTotalWeight = $self->getCalculatedTotalWeight();
    return 1 if (fcmp($TotalWeight, $CalculatedTotalWeight) == 0);
    return 0;
}

#========================================================================================
# function     getMeasurementsToShippingLabel
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShippingLabel->getMeasurementsToShippingLabel();
#----------------------------------------------------------------------------------------
# description  Takes the measures ( lenght, height and width ) from the product in a
#               order ( LineItemContainer ) and convert them to the shipping label units
#               ( Example: convert millimetre to inchs )<br>
#               This work is done only if the order has one item (!)
#----------------------------------------------------------------------------------------
# return       $hReturn | The measurement values converted to the correct delivery unit | refhash
#========================================================================================
sub getMeasurementsToShippingLabel {
    my $self               = shift;

    my $LineItemContainer = $self->get('LineItemContainer');
    my $LineItemProducts = $LineItemContainer->childrenByClass('LineItemProduct');
    my $hReturn = {};

    # If there are more than one product, it makes no sense to get the dimensions
    if ( scalar @$LineItemProducts == 1 && $LineItemProducts->[0]->get('Quantity') == 1 ) {

        my @ValueNames = qw( Length Height Width );

        # Retrieve the delivery measures of the product
        my $hValues = $self->_getParameters( $LineItemProducts->[0], \@ValueNames );

        # We get the names for existing measures in the product
        my @Keys = keys %$hValues ;

        # How many measures are defined ?
        if ( scalar @Keys >= 1 ) {

            # Continue if the product has some measures

            my $Shop = $self->getSite() ;

            # We get the $DeliveryDimensionUnitInch shop flag in order to know if the MBO shows
            # to the merchant the delivery measures in inch or millimetres (!)
            #   See MBO > Products > Products > Delivery details > Dimensions

            my $DeliveryDimensionUnitInch = $Shop->get( 'DeliveryDimensionUnitInch' ) ;

            # And also we need to know the units system for this delivery

            my $SizeUnit = $self->get('SizeUnit');

            if ( $DeliveryDimensionUnitInch and $SizeUnit->alias() ne 'inch' ) {

                # If the units entered are inchs in the MBO but not in the order,
                # we have to recalculate the units

                $hValues = $self->_recalculateDimensions(
                                $hValues,
                                LoadObjectByPath( '/Units/metre/inch' ),
                                $SizeUnit
                            ) ;
            }

            if ( not $DeliveryDimensionUnitInch and $SizeUnit->alias() ne 'millimetre' ) {

                # If the units entered are millimetres in the MBO but not in the order,
                # we have to recalculate the units

                $hValues = $self->_recalculateDimensions(
                                $hValues,
                                LoadObjectByPath( '/Units/metre/millimetre' ),
                                $SizeUnit
                            ) ;
            }

            # Mapping of the measures to the shipping label measures attributes
            $hReturn->{'ParcelDepth'}  = $hValues->{'Height'} if defined $hValues->{'Height'} ;
            $hReturn->{'ParcelLength'} = $hValues->{'Length'} if defined $hValues->{'Length'} ;
            $hReturn->{'ParcelWidth'}  = $hValues->{'Width'} if defined $hValues->{'Width'} ;
        }
    }

    return $hReturn ;
}

#========================================================================================
# function     _recalculateDimensions
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShippingLabel->_recalculateDimensions( $hValues, $SourceUnit, $TargetUnit );
#----------------------------------------------------------------------------------------
# description  This takes a hash of measures and do the conversion from a source units
#               system ( product ) to a target unit system ( delivery ).
#----------------------------------------------------------------------------------------
# input        $hValues | hash of dimension values ( length, width, ... ) | refhash
# input        $SourceUnit | Source UnitOfMeasurement object | object
# input        $TargetUnit | Target UnitOfMeasurement object | object
# return       $hValues | input hash of measurement values converted to the target unit | refhash
#========================================================================================
sub _recalculateDimensions {
    my $self = shift;
    my ( $hValues, $SourceUnit, $TargetUnit ) = @_;

    my @Measures = keys %$hValues ;

    # We will use a formatter to fix float number to 2 decimals
    my $Formatter = DE_EPAGES::Core::API::Object::NumberFormat->new() ;

    # And now start to convert the measures ...

    foreach my $MeasureName ( @Measures ) {

        my $Value = $hValues->{ $MeasureName } ;

        # First we apply the factor to convert to metre

        $Value *= $SourceUnit->get('Factor') ;

        # And now we convert to the delivery units

        $Value /= $TargetUnit->get('Factor') ;

        # Rounding ( if float ) to 2 decimals
        $Value = $Formatter->round( $Value, 2 ) ;

        # Update the converted value
        $hValues->{ $MeasureName } = $Value ;
    }

    return $hValues ;
}

#========================================================================================
# function     _getParameter
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getParameter( $LineItemProduct, $hValueNames );
#----------------------------------------------------------------------------------------
# input        $LineItemProduct | LineItemProduct Object | object
# input        $hValueNames | names of the parameters from LineItemProduct to get | ref.hash
#----------------------------------------------------------------------------------------
# description  This wrapps the get() for a list of parameters to handle in a generic way
#               the 'NoResults_Attribute' Error.
#               We consider the possibility the product could be a variation, so if the
#               request attribute is not set in the variation we try to get it from its
#               superproduct ...
#----------------------------------------------------------------------------------------
# return       $hValue | the Values from the shippinglable | href
#========================================================================================
sub _getParameters {
    my $self = shift;
    my ( $LineItemProduct, $hValueNames ) = @_;

    TestObject('LineItemProduct',$LineItemProduct,'DE_EPAGES::Product::API::Object::LineItemProduct');
    my $ProductID = $LineItemProduct->get('ProductID');
    ExistsObject($ProductID);
    my $Product = LoadObject($ProductID);
    my $hValues;
    foreach my $ValueName ( @{$hValueNames} ) {
        eval { $hValues->{$ValueName} = $Product->getWithParent($ValueName); };
        if ( ExistsError() ) {
            my $Error = GetError();
            Error($Error) unless ( $Error->code eq 'NoResults_Attribute' );
        }
    }
    return $hValues;
}

1;
