#==============================================================================
# package      DE_EPAGES::Forum::UI::BlogPost
# state        private
#==============================================================================
package DE_EPAGES::Forum::UI::BlogPost;
use base DE_EPAGES::Shop::UI::Object;

use strict;
use DE_EPAGES::Core::API::EpagesConfig qw ( EpagesConfig );
use DE_EPAGES::Core::API::Error qw ( GetError ExistsError Error);
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Forum::API::Constants qw (POST_DELAY);
use DE_EPAGES::Forum::API::Mail qw ( SendNewPostMail );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias );
use DE_EPAGES::Search::API::SearchDefinition qw (GetSearchDefinition);
use DE_EPAGES::SEO::API::SEO qw ( PerformSEOCheckAndUpdateObject );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );


sub Save {
    my $self = shift;
    my ($Servlet) = @_;
    my $BlogPost = $Servlet->object;
    $Servlet->form->form($BlogPost, 'EditBlogPostForm');
    # save the main content of the category
    eval {
        $self->SUPER::Save($Servlet);
    };
    if (ExistsError()) {
        my $Error = GetError();
        Error($Error) unless ($Error->code eq 'INVALID_CHARACTER' ||
                              $Error->code eq 'SHORTURL_ALREADY_EXISTS' ||
                              $Error->code eq 'ShortURL_NOT_ALLOWED');
        $Servlet->form->executeFormError({
            'Reason'=> $Error->code,
            'LoopName' => 'Save',
            'Name'  => 'URI',
        });
    }

    # throw changed-hook
    TriggerHook('UI_AfterSaveContentObject', {'Object' => $BlogPost});
    return;
}

sub NewObject {
    my $self = shift;
    my ($Servlet, $Parent, $Class, $hAttributes) = @_;

    my $Form = $Servlet->form;
    my $Object = $Servlet->object;
    $hAttributes = {
        defined $hAttributes ? %$hAttributes : (),
        %{$Form->form($Object, 'NewObject')},
        'User' => $Servlet->user,   # is always valid user as is has to be a MBO merchant
        'RemoteIP' => defined $Servlet->serverVariables->{'REMOTE_ADDR'} ? $Servlet->serverVariables->{'REMOTE_ADDR'} : '',
        'IsVisible' => 1,
        'Thread' => 0
    };

    my $ReturnObject = $self->SUPER::NewObject($Servlet, $Parent, $Class, $hAttributes);
    # EPG-12450 : set thread id to self id
    $ReturnObject->set({'Thread' => $Parent});

    return $ReturnObject;
}

# override implementation of forumpost's _AddReply
sub AddReply {
    my $self = shift;
    my ($Servlet, $DisableReCaptcha) = @_;

    $self->GoogleCaptcha($Servlet, $DisableReCaptcha);

    my $Form = $Servlet->form;
    my $BlogPost = $Servlet->object;
    my $NumReplies = scalar @{$Servlet->object->get('Replies')};
    my $MaxNumBlogReplies = EpagesConfig()->get( 'DE_EPAGES::Forum', 'MaxNumBlogReplies', '1000' );

    # if there's no user, the ip-check is done with the remote ip and the object is inserted
    # without an user (userid attribute can be null)
    my $User = $Servlet->user;
    $User = undef if defined $User && $User->get('IsAnonymous');

    my $hValues = $Form->form($BlogPost, 'AddComment');
    $hValues->{'Spam'} = 2 if defined $User && $User->get('IsMerchant');

    # do not add reply if thread is closed, unless the current user is an admin or you reached the max allowed replies
    return if ($BlogPost->get('IsClosed') and not(defined $User and $User->get('IsMerchant')) or ($NumReplies >= $MaxNumBlogReplies));

    my $Blog = $BlogPost->parent;
    my $RemoteIP = $Servlet->serverVariables->{'REMOTE_ADDR'};
    $Servlet->form->executeFormError({
        'Form'      => 'AddReply',
        'Reason'    => 'UNDERRUN_POST_DELAY',
        'Name'      => 'PostDelay',
        'Value'     => undef,
        'PostDelay' => POST_DELAY,
    }) unless $Blog->checkPostDelay($User, $RemoteIP);

    my $NewPost = $BlogPost->insertChild('ForumPost', {
        defined $hValues ? %$hValues : (),
        'Subject' => $hValues->{'Subject'},
        'Message' => $hValues->{'Message'},
        'IsVisible' => 1,
        'RemoteIP' => $RemoteIP,
        'User' => $User,
        'Forum' => $Blog,
        'Thread' => undef
    });

    # send info mail for new post
    if(!defined $User || !$User->get('IsMerchant')) {
        eval { SendNewPostMail($NewPost) };
        if (ExistsError()) {
            my $Error = GetError();
            LogError($Error);
        }
    }

    return $NewPost;
}

sub SaveList {
    my $self = shift;
    my ($Servlet) = @_;

    # This changeAction is called in MBO and we want no reCaptcha on the MBO
    my $DisableReCaptcha = 1;

    $self->AddReply($Servlet, $DisableReCaptcha);
    return;
}

sub ViewSEOCheck {
    my $self = shift;
    my ($Servlet) = @_;

    my $Object = $Servlet->object;
    my $hValues = $Object->get('CalculatedSEOCheckValues');
    PerformSEOCheckAndUpdateObject($Object);

    $Servlet->vars('ResponseContent', $hValues);

    return $self->SUPER::View($Servlet);
}

sub View {
    my $self = shift;
    my ($Servlet) = @_;

    my $NumReplies = scalar @{$Servlet->object->get('Replies')};
    my $MaxNumBlogReplies = EpagesConfig()->get('DE_EPAGES::Forum', 'MaxNumBlogReplies', '1000');

    my $TooManyReplies = ($NumReplies >= $MaxNumBlogReplies) ? 1 : 0;

    $Servlet->vars('TooManyReplies', $TooManyReplies);

    my $Forum = $Servlet->object;
    my $SearchName = "BlogReplies";
    my $SearchDefinition = GetSearchDefinition(LoadClassByAlias('ForumPost'), $SearchName);
    my %Options = (
        'Servlet' => $Servlet,
        'SearchDefinition' => $SearchDefinition,
        'Parameters' => {
            'ForumID'   => $Forum->id,
            'IsVisible'   => 1,
        },
    );

    return DE_EPAGES::Search::API::Object::PagerSearch->new(\%Options)->page;
}

1;
