#==============================================================================
# package      DE_EPAGES::Forum::UI::Forum
# state        private
#==============================================================================
package DE_EPAGES::Forum::UI::Forum;
use base qw ( DE_EPAGES::Shop::UI::Object Exporter );

use strict;
use DateTime;
use DE_EPAGES::Core::API::Error qw ( GetError ExistsError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Forum::API::Constants qw (POST_DELAY);
use DE_EPAGES::Forum::API::Mail qw ( SendNewPostMail );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias );
use DE_EPAGES::Search::API::Object::PagerSearch qw (ReplaceWildcards);
use DE_EPAGES::Search::API::SearchDefinition qw (GetSearchDefinition);


use constant TIME => 86399;

sub View {
    my $self = shift;
    my ($Servlet) = @_;

    my $OverrideLocale = $Servlet->params('OverrideLocale');
    if (defined $OverrideLocale) {
        $Servlet->setLocale($OverrideLocale, 0);
        $Servlet->params('ViewAction', 'View');
    }

    return $self->_searchThreads( $Servlet, $Servlet->isEditorMode ? 0 : 1, 1 );
}

sub MBOViewChildObjects {
    my $self = shift;
    my ($Servlet) = @_;

    return $self->_searchThreads( $Servlet, 0, 0 );
}

sub _searchThreads {
    my $self = shift;
    my ($Servlet, $OnlyVisibleThreads, $OrderDesc) = @_;

    my $SearchName = $OnlyVisibleThreads ? 'SF-ForumThreads' : 'MBO-ForumThreads';

    my $Forum = $Servlet->object;

    my $SearchDefinition = GetSearchDefinition(LoadClassByAlias('ForumPost'), $SearchName);
    my %Options = (
        'Servlet' => $Servlet,
        'SearchDefinition' => $SearchDefinition,
        'OrderBy' => 'LastPostID',
        'OrderDesc' => $OrderDesc ? 1 : 0,
        'Parameters' => {
            'ForumID'   => $Forum->id,
        },
    );

    return DE_EPAGES::Search::API::Object::PagerSearch->new(\%Options)->page;
}

sub CheckFormInput {
    my $self = shift;
    my ($Form, $FormElementName) = @_;

    my $FormElementValue = $Form->value($FormElementName);
    if(not $FormElementValue or $FormElementValue !~ /\S+/) {
        $Form->addFormError( {
            'Form'      => 'AddThread',
            'Reason'    => 'STRING_EMPTY',
            'Name'      => $FormElementName,
            'Value'     => undef,
        });
        return 0;
    }
    return 1;
}

sub AddThread {
    my $self = shift;
    my ($Servlet, $DisableReCaptcha) = @_;
    my $Form = $Servlet->form;

    $self->GoogleCaptcha($Servlet, $DisableReCaptcha);

    #spam check - check if field senderName has value,
    #this field is only for spam protection it is hidden for the customer

    return if $Form->value('SenderName');

    my $Subject = $Form->value('Subject');
    my $Message = $Form->value('Message');

    if($self->CheckFormInput($Form, 'Subject') and $self->CheckFormInput($Form, 'Message')) {

        # do not add anonymous users to the post's User attribute
        my $User = $Servlet->user;
        $User = undef if defined $User && $User->get('IsAnonymous');

        my $Forum = $Servlet->object;

        # check input values
        $Servlet->form->form($Forum, 'NewObject');

        # check if user is allowed to post again
        my $RemoteIP = $Servlet->serverVariables->{'REMOTE_ADDR'};
        if ($Forum->checkPostDelay($User, $RemoteIP)) {
            # only merchants are allowed to close posts
            my $IsClosed = defined $User && $User->get('IsMerchant') ? $Form->value('IsClosed') : 0;
            my $TagNamesString = $Form->value('TagNamesString');
            my $hAttributes = {
                'Subject'           => $Subject,
                'Message'           => $Message,
                'IsVisible'         => 1,
                'RemoteIP'          => $RemoteIP,
                'User'              => $User,
                'IsClosed'          => defined $IsClosed ? $IsClosed : 0,
                'TagNamesString'    => $TagNamesString,
            };
            # a post given by a merchant is never spam, hopefully
            $hAttributes->{'Spam'} = 2 if (defined $User && $User->get('IsMerchant'));
            my $ReturnObject = $Forum->insertChild($Forum->getPostClassName, $hAttributes);

            # send info mail for new post
            if(!defined $User || !$User->get('IsMerchant')) {
            eval { SendNewPostMail($ReturnObject) };
                if (ExistsError()) {
                    my $Error = GetError();
                    LogError($Error);
                }
            }
            return $ReturnObject;
        } else {
            $Servlet->form->addFormError({
                'Form'      => 'AddThread',
                'Reason'    => 'UNDERRUN_POST_DELAY',
                'Name'      => 'PostDelay',
                'Value'     => undef,
                'PostDelay' => POST_DELAY,
            });
        }
    }

    $Servlet->form->executeFormError;
    return;
}

sub SaveList {
    my $self = shift;
    my ($Servlet) = @_;

    # This changeAction is called in MBO and we want no reCaptcha on the MBO
    my $DisableReCaptcha = 1;

    $self->AddThread($Servlet, $DisableReCaptcha);
    return;
}

sub MBOSearchPosts {
    my $self = shift;
    my ($Servlet) = @_;

    my $SearchDefinition = GetSearchDefinition(LoadClassByAlias('Object'), 'MBO-SearchPosts');
    return $self->_searchPosts($Servlet, $SearchDefinition);
}

sub _searchPosts {
    my $self = shift;
    my ($Servlet, $SearchDefinition) = @_;

    my $Forum = $Servlet->object;
    my $SearchForm = $Servlet->form->form($Forum, 'MBOSearchPosts');
    delete $SearchForm->{$_} foreach grep { not defined $SearchForm->{$_} } keys %$SearchForm;
    $SearchForm->{'SearchString'} = $SearchForm->{'WildSearchString'} if defined $SearchForm->{'WildSearchString'};

    my %Options = (
        'Servlet' => $Servlet,
        'SearchDefinition' => $SearchDefinition,
        'Parameters' => {
            'Shop.ID'   => $Forum->getSite()->id,
            'PostClassIDs' => [ LoadClassByAlias('ForumPost')->id
                , LoadClassByAlias('GuestBookPost')->id
                , LoadClassByAlias('BlogPost')->id ],
        },
        'UriParameters'  => {},
    );

    my $Form = $Servlet->form;
    foreach my $Parameter (keys %$SearchForm) {
        if ($Parameter =~ /String/) {
            # replace * with %
            $Options{'Parameters'}{$Parameter} = ReplaceWildcards($SearchForm->{$Parameter});
        } elsif ($Parameter =~ /ToDate/ && defined $SearchForm->{$Parameter}) {
            # ToDate = end of day 23:59:59
            # plus 1 day minus 1 second
            $Options{'Parameters'}{$Parameter} = DateTime->from_epoch( epoch => $SearchForm->{$Parameter}->epoch + TIME );
        } else {
            $Options{'Parameters'}{$Parameter} = $SearchForm->{$Parameter};
        }
        $Options{'UriParameters'}{$Parameter} = $Form->value($Parameter);
    }

    $Servlet->vars('SEARCH', 1);
    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new(\%Options);
    return $Search->page;
}

sub SearchPosts {
    my $self = shift;
    my ($Servlet) = @_;
    my $Forum = $Servlet->object;
    my $SearchForm = $Servlet->form->form($Forum, 'SearchPosts');
    delete $SearchForm->{$_} foreach grep { not defined $SearchForm->{$_} } keys %$SearchForm;

    my $SearchDefinition = GetSearchDefinition(LoadClassByAlias('ForumPost'), 'SF-SearchPosts');
    my %Options = (
        'Servlet' => $Servlet,
        'Object'  => $Forum,
        'SearchDefinition' => $SearchDefinition,
        'Parameters' => {
            'IsVisible'  => 1,
            'Shop.ID'   => $Forum->getSite()->id,
        },
        'UriParameters'  => {},
    );
    my $Form = $Servlet->form;
    foreach my $Parameter (keys %$SearchForm) {
        if ($Parameter =~ /String/) {
            # replace * with %
            $Options{'Parameters'}{$Parameter} = ReplaceWildcards($SearchForm->{$Parameter});
        } elsif ($Parameter =~ /ToDate/ && defined $SearchForm->{$Parameter}) {
            # ToDate = end of day 23:59:59
            # plus 1 day minus 1 second
            $Options{'Parameters'}{$Parameter} = DateTime->from_epoch( epoch => $SearchForm->{$Parameter}->epoch + TIME );
        } else {
            $Options{'Parameters'}{$Parameter} = $SearchForm->{$Parameter};
        }
        $Options{'UriParameters'}{$Parameter} = $Form->value($Parameter);
    }

    $Servlet->vars('SEARCH', 1);
    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new(\%Options);
    return $Search->page;
}

sub RSS {
    my $self = shift;
    my ($Servlet) = @_;

    $Servlet->vars('ContentType', 'application/rss+xml');

    my $Forum = $Servlet->object;
    my $SearchDefinition = GetSearchDefinition(LoadClassByAlias('ForumPost'), 'SF-SearchPosts');
    my %Options = (
        'ObjectID'  => $Forum->id,
        'SearchDefinition' => $SearchDefinition,
        'Parameters' => {
            'ForumID' => $Forum->id,
            'IsVisible' => 1,
        },
        'OrderBy' => 'CreationDate',
        'OrderDesc' => 1,
        'PageSize' => 20,
    );
    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new(\%Options);

    return $Search->page;
}

1;
