#==============================================================================
# package      DE_EPAGES::Forum::UI::ForumPost
# state        private
#==============================================================================
package DE_EPAGES::Forum::UI::ForumPost;
use base DE_EPAGES::Shop::UI::Object;

use strict;
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Core::API::EpagesConfig qw ( EpagesConfig );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError);
use DE_EPAGES::Forum::API::Constants qw (POST_DELAY);
use DE_EPAGES::Forum::API::Mail qw ( SendNewPostMail );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias );
use DE_EPAGES::Permission::API::Permission qw ( HaveSameSite );
use DE_EPAGES::Search::API::SearchDefinition qw ( GetSearchDefinition );


sub View {
    my $self = shift;
    my ($Servlet) = @_;

    my $PostsCount = $Servlet->object->get('PostsCount');
    my $MaxNumForumReplies = EpagesConfig()->get('DE_EPAGES::Forum', 'MaxNumForumReplies', '1000');

    my $TooManyReplies = ($PostsCount >= $MaxNumForumReplies) ? 1 : 0;

    $Servlet->vars('TooManyReplies', $TooManyReplies);

    return $self->_searchPosts( $Servlet, $Servlet->isEditorMode ? 0 : 1 );
}

sub MBOViewGeneral {
    my $self = shift;
    my ($Servlet) = @_;

    return $self->_searchPosts( $Servlet, 0 );
}

sub _searchPosts {
    my $self = shift;
    my ($Servlet, $OnlyVisiblePosts) = @_;

    my $Thread = $Servlet->object;

    my $SearchDefinition = GetSearchDefinition(LoadClassByAlias('ForumPost'), 'ThreadPosts');
    my %Options = (
        'Servlet' => $Servlet,
        'SearchDefinition' => $SearchDefinition,
        'OrderBy' => 'CreationDate',
        'Parameters' => {
            'ThreadID' => $Thread->id,
        },
    );
    $Options{'IsVisible'} = 1 if $OnlyVisiblePosts;

    return DE_EPAGES::Search::API::Object::PagerSearch->new(\%Options)->page;
}

sub _AddReply {
    my $self = shift;
    my ($Servlet, $Subject, $Message, $hValues) = @_;

    my $ParentPost = $Servlet->object;

    # first reply or reply of another reply?
    my $Thread = $ParentPost->get('Thread');
    if(not defined $Thread or $Thread->id == $ParentPost->id) {
        $Thread = $ParentPost->getSite()->loadObject($ParentPost->get('ForumPostID'), 'ForumPost');
    }

    # do not add anonymous users to the post's User attribute
    my $User = $Servlet->user;
    $User = undef if defined $User && $User->get('IsAnonymous');

    my $Forum = $ParentPost->get('Forum');
    # do not add reply if thread is closed, unless the current user is an admin
    return undef if $Thread->get('IsClosed') and not(defined $User and $User->get('IsMerchant'));

    $Servlet->form->executeFormError( {
            'Form'      => 'AddThread',
            'Reason'    => 'STRING_EMPTY',
            'Name'      => 'Subject',
            'Value'     => undef,
    }) unless $Subject =~ /\S/;

    $Servlet->form->executeFormError( {
            'Form'      => 'AddThread',
            'Reason'    => 'STRING_EMPTY',
            'Name'      => 'Message',
            'Value'     => undef,
    }) unless $Message =~ /\S/;

    # check Site
    unless (HaveSameSite($ParentPost, $Thread)) {
        die('Foreign ParentPost (' . $ParentPost->id . ') used in Shop (' . $Thread->getSite()->id . ')');
    }

    my $RemoteIP = $Servlet->serverVariables->{'REMOTE_ADDR'};
    $Servlet->form->executeFormError({
        'Form'      => 'AddReply',
        'Reason'    => 'UNDERRUN_POST_DELAY',
        'Name'      => 'PostDelay',
        'Value'     => undef,
        'PostDelay' => POST_DELAY,
    }) unless $Forum->checkPostDelay($User, $RemoteIP);

    my $NewPost = $ParentPost->insertChild('ForumPost', {
        defined $hValues ? %$hValues : (),
        'Subject' => $Subject,
        'Message' => $Message,
        'IsVisible' => 1,
        'RemoteIP' => $RemoteIP,
        'User' => $User,
        'Forum' => $Forum,
        'Thread' => $Thread
    });

    # send info mail for new post
    if(!defined $User || !$User->get('IsMerchant')) {
        eval { SendNewPostMail($NewPost) };
        if (ExistsError()) {
            my $Error = GetError();
            LogError($Error);
        }
    }
    return $NewPost;
}

sub AddReply {
    my $self = shift;
    my ($Servlet) = @_;
    my $Form = $Servlet->form;

    $self->GoogleCaptcha($Servlet);

    my $PostsCount = $Servlet->object->get('PostsCount');
    my $MaxNumForumReplies = EpagesConfig()->get('DE_EPAGES::Forum', 'MaxNumForumReplies', '1000');

    #check if field senderName has value (spamCheck) or the limit of maximum replies is reached
    return if ($Form->value('SenderName') or ($PostsCount >= $MaxNumForumReplies));

    my $Forum = $Servlet->object->get('Forum');
    my $User = $Servlet->user;
    my %Values;
    $Values{'Spam'} = 2 if defined $User && $User->get('IsMerchant');
    $Servlet->form->form($Forum, 'NewObject');
    return $self->_AddReply($Servlet, $Form->value('Subject'), $Form->value('Message'), \%Values);
}

sub Save {
    my $self = shift;
    my ($Servlet) = @_;
    my $Form = $Servlet->form;
    my $Subject = $Form->value('Subject');
    if (defined $Subject && $Subject eq '') {
        $Form->executeFormError({
            'Name' => 'Subject',
            'Form' => 'SaveThread',
            'Reason' => 'STRING_EMPTY'
        });
    }
    $self->SUPER::Save($Servlet);
    my $Forum = $Servlet->object->get('Forum');

    my $NewMessage = $Form->value('NewMessage');
    my $NewSubject = $Form->value('NewSubject');

    # do only create a new reply when all needed parameters are given
    if(defined $NewMessage && defined $NewSubject) {
        if ($NewMessage =~ /\S+/ && $NewSubject =~ /\S+/) {
            $Servlet->form->form($Forum, 'AddReply');
            $self->_AddReply($Servlet, $NewSubject, $NewMessage, {'Spam' => 2});
        } else {
            $Servlet->form->addFormError( {
                'Form'      => 'AddReply',
                'Reason'    => 'STRING_EMPTY',
                'Name'      => 'NewMessage',
                'Value'     => undef,
            }) if $NewMessage =~ /\S+/ && $NewSubject !~ /\S+/ ;
        }
    }
    $Servlet->form->executeFormError;
}

sub ReportSpam {
    my $self = shift;
    my ($Servlet) = @_;
    my $ForumPost = $Servlet->object;

    $ForumPost->set({'Spam' => 1}) if ($ForumPost->get('Spam') == 0);
    return;
}

sub SetElementsNoSpam {
    my $self = shift;
    my ($Servlet) = @_;
    return $self->SetAttributeOnListedObjects($Servlet, 'Spam', 2);
}

sub NewObject {
    my $self = shift;
    my ($Servlet, $Parent, $Class, $hAttributes) = @_;

    my $Form = $Servlet->form;

    #spam check - check if field senderName has value,
    #this field is only for spam protection it is hidden for the customer
    return if $Form->value('SenderName');

    my $Object = $Servlet->object;

    $hAttributes = {
        defined $hAttributes ? %$hAttributes : (),
        %{$Form->form($Object, 'NewObject')},
        'User' => defined $Servlet->user ? $Servlet->user : undef,
        'RemoteIP' => defined $Servlet->serverVariables->{'REMOTE_ADDR'} ? $Servlet->serverVariables->{'REMOTE_ADDR'} : '',
        'IsVisible' => 1,
    };
    # delete Spam protection field is no longer needed and can not be saved inside the database
    delete $hAttributes->{'SenderName'};

    return $self->SUPER::NewObject($Servlet, $Parent, $Class, $hAttributes);
}

1;
