#========================================================================================
# package      DE_EPAGES::GoogleAdWords::Hooks::Shop::OnBeforeUpdateShop
# state        public
# description  Imports default shop data into a new shop when it is created.
#========================================================================================
package DE_EPAGES::GoogleAdWords::Hooks::Shop;

use strict;

use DE_EPAGES::AppStore::API::Object::DeveloperAppsClient;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::EPagesJ::API::Constants  qw (
    SHOPS_RESOURCE_PATH
    SCRIPT_TAGS_RESOURCE_PATH
    MY_SCRIPT_TAGS_URL_QUERY
);
use DE_EPAGES::GoogleAdWords::API::Constants  qw (
    GOOGLE_AD_WORDS_APP_SCOPES
    GOOGLE_AD_WORDS_SCRIPT_URL_QUERY
);
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );

#========================================================================================
# function     OnBeforeUpdateShop
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnBeforeUpdateShop( $hParams );
#----------------------------------------------------------------------------------------
# description  check if GoogleAdWordsHTMLCode has changed,
#               parse the related GoogleAdWordsID out of it
#               and regeister the script-tag via the REST-API
#----------------------------------------------------------------------------------------
# input        $hParams | hook parameters
#               <ul>
#                 <li>Object        - shop object          - object
#                 <li>ObjectID      - shop object id            - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul> | ref.hash.*
#========================================================================================
sub OnBeforeUpdateShop {
    my ($hParams) = @_;
    my $Shop = $hParams->{'Object'};

    if ($Shop->get('IsUnityRegistered')) {
        my $hValues = $hParams->{'Values'};
        if (exists $hValues->{'GoogleAdWordsHTMLCode'}) {
            my $GoogleAdWordsHTMLCode = $hValues->{'GoogleAdWordsHTMLCode'};
            my $hInfoShop = $Shop->get(['UnityGoogleAdWordsAppGUID', 'WebServerScriptNamePart', 'WebServerScriptUrlSSL']);
            # do not proceed without app and no adWords code
            if (!defined $GoogleAdWordsHTMLCode && !defined $hInfoShop->{'UnityGoogleAdWordsAppGUID'}) {
                $Shop->set({'GoogleAdWordsID' => undef});
                return;
            }

            # get app to communicate with the epagesJ REST API
            my $AppsClient = DE_EPAGES::AppStore::API::Object::DeveloperAppsClient->create({'Shop' => $Shop});
            my $GoogleAdWordsApp = _getGoogleAdWordsApp($AppsClient, $Shop, $hInfoShop->{'UnityGoogleAdWordsAppGUID'});

            # get existing script tag, if any
            my $ScriptTagUrl = LoadRootObject->get('EPagesJBaseURL') . SHOPS_RESOURCE_PATH . $hInfoShop->{'WebServerScriptNamePart'} . SCRIPT_TAGS_RESOURCE_PATH;
            my $GoogleAdwordsScriptTagUrl = $hInfoShop->{'WebServerScriptUrlSSL'} . GOOGLE_AD_WORDS_SCRIPT_URL_QUERY;
            my $hHeaders = _requestHeaders($GoogleAdWordsApp);
            my ($IsGoogleAdWordsScriptTagPresent, $hGoogleAdWordsScriptTag)
                = _isGoogleAdWordsScriptTagPresent($AppsClient, $ScriptTagUrl, $hHeaders,
                    $GoogleAdwordsScriptTagUrl, $hInfoShop->{'UnityGoogleAdWordsAppGUID'});

            # parse id from Google AdWords script
            my $GoogleAdWordsID = undef;
            if (defined $GoogleAdWordsHTMLCode) {
                # parse id=xxx" or id=xxx& and gtag('config', 'xxx');
                # check that both ids are identical
                if ($GoogleAdWordsHTMLCode =~ m/id=([^\"&]*)[\"&][\s\S]*gtag\(\'config\', \'([^']*)\'\)/
                    && $1 eq $2) {
                    $GoogleAdWordsID = $1;
                } else {
                    # throw error if not id found while parsing --> either wrong merchant input
                    # or google ad words script format changed --> parsing has fo be adapted
                    Error('NoGoogleAdWordsIDFound');
                }
                unless ($IsGoogleAdWordsScriptTagPresent) {
                    # post new script tag, in case there has not been any so far
                    my $hContent = { 'scriptUrl' => $GoogleAdwordsScriptTagUrl };
                    my $EncodedContent = $AppsClient->encodeRequest( $hContent );
                    $AppsClient->client->post( $ScriptTagUrl, $hHeaders, $EncodedContent );
                }
            } else {
                # no GoogleAdWordsHTMLCode given any more
                # delete existing script-tag
                $AppsClient->client->delete( $ScriptTagUrl . '/' . $hGoogleAdWordsScriptTag->{'scriptTagId'}, $hHeaders )
                    if $IsGoogleAdWordsScriptTagPresent;
                # delete app
                $AppsClient->deleteApp($GoogleAdWordsApp->{'privateAppId'});
                $Shop->set({ 'UnityGoogleAdWordsAppGUID' => undef });
            }

            $Shop->set({'GoogleAdWordsID' => $GoogleAdWordsID});
        }
    }

    return;
}

sub _isGoogleAdWordsScriptTagPresent {
    my ($AppsClient, $ScriptTagUrl, $hHeaders,
        $GoogleAdwordsScriptTagUrl, $UnityGoogleAdWordsAppGUID) = @_;

    # do not request script-tags api in case there has not been any app yet
    return (0, undef) unless defined $UnityGoogleAdWordsAppGUID;

    # get all script tags created by the GoogleAdWords API
    my $RequestURL = $ScriptTagUrl . MY_SCRIPT_TAGS_URL_QUERY;
    my $Response = $AppsClient->client->get( $RequestURL, $hHeaders );
    my $hScripTags = $AppsClient->decodeResponse($Response->decoded_content);

    #check if Google ad words script tag URL is already present
    my $IsGoogleAdWordsScriptTagPresent = 0;
    my $hGoogleAdWordsScriptTag;
    foreach my $hScriptTag (@{ $hScripTags->{'items'} }) {
        if ($hScriptTag->{'scriptUrl'} eq $GoogleAdwordsScriptTagUrl) {
            $IsGoogleAdWordsScriptTagPresent = 1;
            $hGoogleAdWordsScriptTag = $hScriptTag;
            last;
        }
    }

    return ($IsGoogleAdWordsScriptTagPresent, $hGoogleAdWordsScriptTag);
}

sub _getGoogleAdWordsApp {
    my ($AppsClient, $Shop, $UnityGoogleAdWordsAppGUID) = @_;

    my $GoogleAdWordsApp;

    if (defined $UnityGoogleAdWordsAppGUID) {
        # get credentials from existing app
        $GoogleAdWordsApp = $AppsClient->getApp($UnityGoogleAdWordsAppGUID);
    } else {
        # create new app, if there has not been any so far
        $GoogleAdWordsApp = $AppsClient->createApp( 'GoogleAdWords', undef, GOOGLE_AD_WORDS_APP_SCOPES );
        $Shop->set({ 'UnityGoogleAdWordsAppGUID' => $GoogleAdWordsApp->{'privateAppId'} });
    }

    return $GoogleAdWordsApp;
}

sub _requestHeaders {
    my ($GoogleAdWordsApp) = @_;

    my $hHeaders = {
        'Authorization' => 'Bearer ' . $GoogleAdWordsApp->{'token'},
        'User-Agent'    => 'epages6 via GoogleAdWords app'
    };

    return $hHeaders;
}

1;
