#========================================================================================
# package      DE_EPAGES::Newsletter::API::Object::NewsletterCampaign
# state        public
# base         DE_EPAGES::Object::API::Object::Object
# description  object interface for NewsletterCampaign.
#========================================================================================
package DE_EPAGES::Newsletter::API::Object::NewsletterCampaign;
use base DE_EPAGES::Object::API::Object::Object;

use strict;
use DE_EPAGES::Core::API::Error qw ( GetError ExistsError Error );
use DE_EPAGES::Core::API::PerlTools qw ( trim isValid GetCurrentDateTime );
use DE_EPAGES::Core::API::TestParameter qw ( TestHash TestEMailAddress TestString );
use DE_EPAGES::Newsletter::API::Constants qw ( NEWSLETTER_ADDRESSEE_NAME_MAXLENGTH );
use DE_EPAGES::Object::API::Factory qw ( Transaction LoadClassByAlias );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $NewsletterCampaign->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'NewsletterCampaign' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $NewsletterCampaign->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        public
#----------------------------------------------------------------------------------------
# syntax       $NewsletterCampaign = DE_EPAGES::Newsletter::API::Object::NewsletterCampaign->insert($hInfo);
# example      my $Class = LoadClassByAlias('NewsletterCampaign');
#               my $NewsletterCampaign = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertNewsletterCampaign' is triggered.
#               Before insert this info the hook OBJ_BeforeInsertNewsletterCampaign is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $NewsletterCampaign | new object | object
# hook         OBJ_InsertNewsletterCampaign | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>NewsletterCampaignID - Identifier of Newsletter Campaign - int
#               </ul>
# hook         OBJ_BeforeInsertNewsletterCampaign | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>NewsletterCampaignID - Identifier of Newsletter Campaign - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    TriggerHook('OBJ_BeforeInsertNewsletterCampaign', { 'hInfo' => $hInfo });

    my $self = $class->SUPER::insert($hInfo);

    $self = bless $self, $class;
    TriggerHook('OBJ_InsertNewsletterCampaign', {'Object' => $self, 'NewsletterCampaignID' => $self->id });
    delete $hInfo->{$_} foreach qw ( Site IsVisible LangID Name Subject SenderName SenderEMail TestEMail );
    return $self;
}

#========================================================================================
# function     delete
# state        public
#----------------------------------------------------------------------------------------
# syntax       $NewsletterCampaign->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteNewsletterCampaign' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteNewsletterCampaign | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>NewsletterCampaignID - Identifier of Newsletter Campaign - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeleteNewsletterCampaign', {'Object' => $self, 'NewsletterCampaignID' => $self->id });
    LoadClassByAlias('Newsletter')->deleteObjects('ByNewsletterCampaignID', [$self->id]);
    LoadClassByAlias('NewsletterAddressee')->deleteObjects('ByNewsletterCampaignID', [$self->id]);
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        public
#----------------------------------------------------------------------------------------
# syntax       $NewsletterCampaign->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateNewsletterCampaign' and 'OBJ_AfterUpdateNewsletterCampaign'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateNewsletterCampaign | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>NewsletterCampaignID - Identifier of Newsletter Campaign - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateNewsletterCampaign | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>NewsletterCampaignID - Identifier of Newsletter Campaign - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set { ##no critic qw(ProhibitAmbiguousNames)
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdateNewsletterCampaign', {'Object'=>$self, 'Values'=> $hValues, 'NewsletterCampaignID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateNewsletterCampaign',  {'Object'=>$self, 'Values'=> $hValues, 'NewsletterCampaignID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     existsAddressee
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Exists = $NewsletterCampaign->existsAddressee($EMailAddress);
#----------------------------------------------------------------------------------------
# description  This function tests if the newsletter campaign own a certain addressee.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new addressee
#                       <ul>
#                         <li>EMail - email address - varchar(255)</li>
#                       </ul> | ref.hash
# return       $Exists | true if the addressee exists | boolean
#========================================================================================
sub existsAddressee {
    my $self = shift;
    my ($EMail) = @_;

    TestString('EMail',$EMail);
    TestEMailAddress('EMail', $EMail);

    my $Class = LoadClassByAlias('NewsletterAddressee');
    my $aAddressees = $Class->findObjects('ByEMailNewsletterCampaignID',[$EMail,$self->id]);

    return scalar @{$aAddressees} ? 1 : 0;
}

#========================================================================================
# function     getAddressee
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $Addresse = $NewsletterCampaign->getAddressee($EMail);
#----------------------------------------------------------------------------------------
# description  This function returns a certain addressee of the newsletter campaign.
#----------------------------------------------------------------------------------------
# input        $EMail | email address | string
# return       $Addressee | addressee exists | DE_EPAGES::Newsletter::API::Object::NewsletterAddressee
#========================================================================================
sub getAddressee {
    my $self = shift;
    my ($EMail) = @_;

    TestString('EMail',$EMail);

    my $Class = LoadClassByAlias('NewsletterAddressee');
    my $aAddressees = $Class->getObjects('ByEMailNewsletterCampaignID', [$EMail,$self->id]);

    return $aAddressees->[0];
}

#========================================================================================
# function     addAddressee
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $Addressee = $NewsletterCampaign->addAddressee($hInfo);
#----------------------------------------------------------------------------------------
# description  This function adds an addressee to the newsletter campaign.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new addressee
#                       <ul>
#                         <li>Name - name of addressee (optional) - varchar(255)</li>
#                         <li>EMail - email address - varchar(255)</li>
#                         <li>User - user object (optional) - object</li>
#                         <li>Unsubscribed - subscription status (optional) - integer</li>
#                         <li>SubscribedInCheckout - subscription is not confirmed and was
#                             done during order process or registration (optional) - boolean</li>
#                       </ul> | ref.hash
# return       $Addressee | addressee object | DE_EPAGES::Newsletter::API::Object::NewsletterAddressee
#========================================================================================
sub addAddressee {
    my $self = shift;
    my ($hInfo) = @_;

    TestHash('AddresseeAttributes', $hInfo, {
                'Name'                =>'String',
                'Unsubscribed'        =>'Integer',
                'SubscribedInCheckout'=>'Integer',
            });

    TestString('AddresseeName', $hInfo->{'Name'}, 0, NEWSLETTER_ADDRESSEE_NAME_MAXLENGTH) if isValid($hInfo->{'Name'});
    TestEMailAddress('AddresseeEMailAddress', $hInfo->{'EMail'});

    my $Class = LoadClassByAlias('NewsletterAddressee');
    return $Class->insertObject({
        'NewsletterCampaign' => $self,
        'Name'     => $hInfo->{'Name'},
        'EMail'    => $hInfo->{'EMail'},
        'User'     => $hInfo->{'User'},
        'Unsubscribed' => $hInfo->{'Unsubscribed'} // 0,
        'SubscribedInCheckout' => $hInfo->{'SubscribedInCheckout'} ? 1 : 0,
        'CreationDate' => GetCurrentDateTime(),
    });
}

#========================================================================================
# function     addCustomer
# state        public
#----------------------------------------------------------------------------------------
# syntax       $NewsletterCampaign->addCustomer($Customer);
#----------------------------------------------------------------------------------------
# description  This function adds a customer to a newsletter campaign.
#----------------------------------------------------------------------------------------
# input        $Customer | customer object | object
#========================================================================================
sub addCustomer {
    my $self = shift;
    my ($Customer) = @_;

    # security check: newsletter campaign and customer should belong to the same shop
    Error('InvalidShop',{ a=>$Customer->getSite()->id, b=>$self->getSite()->id}) unless $Customer->getSite()->id == $self->getSite()->id;

    # count addressee only if restricted add
    my $CountAddressees = 0;
    my $MaxCountAddressees = $self->getSite()->get('MaxNewsletterAddressees');

    if( $MaxCountAddressees != -1 ) {
        $CountAddressees = $self->get('AddresseeCount');
    }

    my $BillingAddress = $Customer->get('BillingAddress');
    my $Users = $Customer->get('Users');

    if (scalar @$Users) {
        my $Counter = 0;
        foreach my $User (@$Users) {
            my $EMail = $User->get('EMail');
            if (defined $EMail) {
                ($EMail) = trim($EMail);
                $EMail = undef if $EMail eq '';
            }
            if ($CountAddressees + $Counter >= $MaxCountAddressees and not $MaxCountAddressees == -1) {
                Error('NumberOfAddresseesExceeded', {
                    'Name'=>'Name',
                    'NewsletterCampaign' => $self->tleHash,
                    'EMail' => $EMail,
                });
            }
            if( $EMail ) {
                if( $self->existsAddressee($EMail) ) {
                    Error('EMailFromTrayInAddresseeList', {
                        'Name'=>'AddCustomerAndUserListFromTray',
                        'NewsletterCampaign' => $self->tleHash,
                        'CustomerNo'   => $Customer->alias,
                        'CustomerName' => defined $BillingAddress ? $BillingAddress->get('DisplayName') : undef,
                        'EMail' => $EMail,
                    });
                    next;
                }
                eval {
                    Transaction(sub {
                        $self->addAddressee({
                            'User'     => $User,
                            'Name'     => $User->get('Name') ||
                                          $BillingAddress->get('DisplayName') ||
                                          '',
                            'EMail'    => $EMail,
                        });
                    });
                    $Counter++;
                };
                if (ExistsError()) {
                    my $Error = GetError();
                    Error($Error);
                    Error('InsertNotSuccessful', {
                        'Name'=>'AddCustomerAndUserListFromTray',
                        'NewsletterCampaign' => $self->tleHash,
                        'CustomerNo'   => $Customer->alias,
                        'CustomerName' => defined $BillingAddress ? $BillingAddress->get('DisplayName') : undef,
                    });
                }
            } else {
                Error('NoEMail', {
                    'Name'=>'AddCustomerAndUserListFromTray',
                    'NewsletterCampaign' => $self->tleHash,
                    'CustomerNo'   => $Customer->alias,
                    'CustomerName' => defined $BillingAddress ? $BillingAddress->get('DisplayName') : undef,
                });
            }
        }
      } else {
        # anonymous user or user has removed his account
        my $EMail = defined $BillingAddress ? $BillingAddress->get('EMail') : undef;
        if (defined $EMail) {
            ($EMail) = trim($EMail);
            $EMail = undef if $EMail eq '';
        }
        if ($CountAddressees >= $MaxCountAddressees and not $MaxCountAddressees == -1) {
            Error('NumberOfAddresseesExceeded', {
                'Name'=>'Name',
                'NewsletterCampaign' => $self->tleHash,
                'EMail' => $EMail,
            });
        }
        if (not defined $EMail) {
            Error('NoEMail', {
                'Name'=>'AddCustomerAndUserListFromTray',
                'NewsletterCampaign' => $self->tleHash,
                'CustomerNo'   => $Customer->alias,
                'CustomerName' => defined $BillingAddress ? $BillingAddress->get('DisplayName') : undef,
            });
            next;
        }
        if( $self->existsAddressee($EMail) ) {
            Error('EMailFromTrayInAddresseeList', {
                'Name'=>'AddCustomerAndUserListFromTray',
                'NewsletterCampaign' => $self->tleHash,
                'CustomerNo'   => $Customer->alias,
                'CustomerName' => defined $BillingAddress ? $BillingAddress->get('DisplayName') : undef,
                'EMail' => $EMail,
            });
            next;
        }
        eval {
            Transaction(sub {
                $self->addAddressee({
                    'Name'     => $BillingAddress->get('DisplayName') || $BillingAddress->get('FullName'),
                    'EMail'    => $EMail,
                });
            });
        };
        if (ExistsError()) {
            Error('InsertNotSuccessful', {
                'Name'=>'AddCustomerAndUserListFromTray',
                'NewsletterCampaign' => $self->tleHash,
                'CustomerNo'   => $Customer->alias,
                'CustomerName' => defined $BillingAddress ? $BillingAddress->get('DisplayName') : undef,
            });
        }
    }

    return;
}

sub addUser{
    my $self = shift;
    my ($User) = @_;

    # security check: newsletter campaign and user should belong to the same shop
    Error('InvalidShop',{ a=>$User->getSite()->id, b=>$self->getSite()->id}) unless $User->getSite()->id == $self->getSite()->id;
    my $EMail = $User->get('EMail');
    if (defined $EMail) {
        ($EMail) = trim($EMail);
        $EMail = undef if $EMail eq '';
    }

    my $MaxCountAddressees = $self->getSite()->get('MaxNewsletterAddressees');
    if ( $MaxCountAddressees != -1 && $self->get('AddresseeCount') + 1 >= $MaxCountAddressees  ) {
        Error('NumberOfAddresseesExceeded', {
            'Name'=>'Name',
            'NewsletterCampaign' => $self->tleHash,
            'EMail' => $EMail,
        });
    }

    if( $EMail) {
        if( $self->existsAddressee($EMail) ) {
            Error('EMailFromTrayInAddresseeList', {
                'Name'=>'AddCustomerAndUserListFromTray',
                'NewsletterCampaign' => $self->tleHash,
                'UserNo'   => $User->alias,
                'UserName' => defined $User->get('Name') ? $User->get('Name') : undef,
                'EMail' => $EMail,
            });
            next;
        }
        eval {
            Transaction(sub {
                $self->addAddressee({
                    'User'     => $User,
                    'Name'     => $User->get('Name') || '',
                    'EMail'    => $EMail,
                });
            });
        };
        if (ExistsError()) {
            my $Error = GetError();
            Error($Error);
            Error('InsertNotSuccessful', {
                'Name'=>'AddCustomerAndUserListFromTray',
                'NewsletterCampaign' => $self->tleHash,
                'UserNo'   => $User->alias,
                'UserName' => defined $User->get('Name') ? $User->get('Name') : undef,
            });
        }
    } else {
        Error('NoEMail', {
            'Name'=>'AddCustomerAndUserListFromTray',
            'NewsletterCampaign' => $self->tleHash,
            'CustomerNo'   => $User->alias,
            'CustomerName' => defined $User->get('Name') ? $User->get('Name') : undef,
        });
    }
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::Newsletter::API::Object::NewsletterCampaign->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;
    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'Site'      => $Parent->parent(),
        'IsVisible' => 0,
    };
}

#========================================================================================
# function     exportXMLElements
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->exportXMLElements($Driver, $hAllAttributes);
#----------------------------------------------------------------------------------------
# description  Exports addressees of the newsletter campaign to the xml SAX driver.
#----------------------------------------------------------------------------------------
# input        $Driver | export driver | object
# input        $hAllAttributes | all attributes of object,
#                                 see $Object-><function allAttributes> | ref.hash
#========================================================================================
sub exportXMLElements {
    my $self = shift;
    my ($Driver, $hAttributes) = @_;

    # export language tags
    $self->SUPER::exportXMLElements($Driver, $hAttributes);

    # do only export child objects if ExportRecursive
    if($Driver->{'ExportRecursive'} && $Driver->exportLevel == 1) {

        $Driver->addObject($_->tagNameXML, $_) foreach sort {
            $a->get('EMail') cmp $b->get('EMail')
        } @{$self->get('Addressees')};
    }

    return;
}

sub importXML {
    my $class = shift;
    my ($Handler, $hElement, $Class, $hAttributes) = @_;

    my $self = $class->SUPER::importXML($Handler, $hElement, $Class, $hAttributes);

    $Handler->pushID('NewsletterCampaign', $self);

    return $self;
}

sub importXMLEnd {
    my $self = shift;
    my ($Handler, $hElement) = @_;

    $Handler->popID('NewsletterCampaign');

    $self->SUPER::importXMLEnd($Handler, $hElement);
    return;
}

sub deleteXML {
    my $class = shift;
    my ($Handler, $hElement, $Class, $hAttributes) = @_;

    my $self = $class->SUPER::deleteXML($Handler, $hElement, $Class, $hAttributes);

    #push object related data only if not deleted
    $Handler->pushID('NewsletterCampaign', $self) if defined $self;

    return $self;
}

sub deleteXMLEnd {
    my $self = shift;
    my ($Handler, $hElement) = @_;

    $Handler->popID('NewsletterCampaign');

    $self->SUPER::deleteXMLEnd($Handler, $hElement);
    return;
}

1;
