#==============================================================================
# package      DE_EPAGES::Newsletter::UI::Site
# state        private
#==============================================================================
package DE_EPAGES::Newsletter::UI::Site;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use DE_EPAGES::Object::API::Factory qw (ExistsObject LoadRootObject);
use DE_EPAGES::Core::API::Scramble qw (Scramble Unscramble);
use DE_EPAGES::Core::API::Error qw ( Error GetError ExistsError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Core::API::PerlTools qw ( CreateSecureCode VerifySecureCode );
##no critic (RequireCorrectImports)
use DE_EPAGES::Newsletter::API::Constants qw ( NEWSLETTER_ADDRESSEE_REQUEST_SUBSCRIBE
                                               NEWSLETTER_ADDRESSEE_REQUEST_UNSUBSCRIBE
                                               NEWSLETTER_ADDRESSEE_SUBSCRIBED
                                               NEWSLETTER_ADDRESSEE_UNSUBSCRIBED );
##use critic

#==============================================================================
# Event handlers
#==============================================================================
sub RequestNewsletter {
    my $self = shift;
    my ($Servlet) = @_;

    $self->GoogleCaptcha($Servlet);

    my $Site = $Servlet->object;
    my $User = $Servlet->user;

    my $Form       = $Servlet->form;
    my $FormValues = $Form->formValues;

    my $IsAnonymous = defined $User ? $User->get('IsAnonymous') : 1;
    my $Name  = ! $IsAnonymous ? $User->get('Name')  : $FormValues->{'Name'}->[0];
    my $EMail = ! $IsAnonymous ? $User->get('EMail') : $FormValues->{'EMail'}->[0];

    $EMail =~ s/\s//g;
    $Servlet->params('EMail', $EMail);

    my $UnSubscribe = $FormValues->{'SetUnsubscribed'}->[0] ? 1 : 0;
    my $UnSubscribeRequest = $UnSubscribe ? NEWSLETTER_ADDRESSEE_REQUEST_UNSUBSCRIBE : NEWSLETTER_ADDRESSEE_REQUEST_SUBSCRIBE;

    # EPC-396 Change newsletter form to become uninteresting for spammers
    if ($Name =~ /http/ or $Name =~ /www/ or $Name =~ /\/\// or $Name =~ /@/) {
        $Form->executeFormError({
            'Reason'=> 'InvalidNewsletterNameFormat',
            'Name'  => 'Name',
        }) ;
        return;
    }

    my $System = LoadRootObject();
    my $MaxLengthName = $System->get('MaxLengthRegistrationNameFields');
    if( length $Name > $MaxLengthName ) {
        $Form->executeFormError({
            'Reason'=> 'AddresseNameTooLong',
            'Name'  => 'Name',
        }) ;
        return;
    }


    if ($EMail !~ /\S/) {
        $Form->executeFormError({
            'Reason'=> 'InvalidEMailFormat',
            'Name'  => 'EMail',
        }) ;
        return;
    }

    my @Campaigns;
    foreach my $ListObjectID (@{$FormValues->{'ListObjectID'}}){
        if ($ListObjectID ne '' and ExistsObject($ListObjectID)){
            my $ListObject = $Site->loadObject($ListObjectID, 'NewsletterCampaign');
            push @Campaigns,$ListObject;
        }
    }

    my $SendMail = 1;
    foreach my $campaign (@Campaigns) {
        eval {
            if ($campaign->existsAddressee($EMail)) {
                my $Addressee = $campaign->getAddressee($EMail);
                $Addressee->set({
                    'Unsubscribed' => $UnSubscribeRequest,
                    'Name'         => $Name,
                });
            } elsif ($UnSubscribe == 0) {
                # create new addressee (user or not user)
                $campaign->addAddressee({
                    'User'         => $IsAnonymous ? undef : $User,
                    'Name'         => $Name,
                    'EMail'        => $EMail,
                    'Unsubscribed' => $UnSubscribeRequest,
                });
            } else {
                $SendMail = 0;
            }
        };
        if (ExistsError()) {
            my $Error = GetError();
            if ( $Error->code eq 'FORMAT_NOT_EMAILADDRESS' ) {
                $Form->executeFormError({
                    'Reason'=> 'InvalidEMailFormat',
                    'Name'  => 'EMail',
                });
            }
            elsif ( $Error->code eq 'STRING_TOO_LONG' ) {
                $Form->executeFormError({
                    'Reason'=> 'AddresseNameTooLong',
                    'Name'  => 'Name',
                });
            }
            else {
                Error($Error);
            }

            return;
        }
    }
    if(scalar @Campaigns == 0){
        $Form->executeFormError({
            'Reason'=> 'NoCampaignSelected',
            'Name'  => 'Campaign',
        }) ;
        return;
    }

    $self->SendConfirmationRequestMail($Servlet, $Name, $EMail, $UnSubscribe, \@Campaigns) if $SendMail;

    $Servlet->vars('CONFIRMATION_REQUEST_MAIL_SENT', 1);

    return;
}

sub ConfirmNewsletter {
    my $self = shift;
    my ($Servlet) = @_;

    my $User = $Servlet->user;
    my $Site = $Servlet->object()->getSite();

    my $IsAnonymous = defined $User ? $User->get('IsAnonymous') : 1;
    my $Name  = !$IsAnonymous ? $User->get('Name')  : Unscramble($Servlet->params('Name'));
    my $EMail = !$IsAnonymous ? $User->get('EMail') : Unscramble($Servlet->params('EMail'));
    $Servlet->params('Name', $Name);
    $Servlet->params('EMail', $EMail);

    my $UnSubscribe = $Servlet->params('Unsubscribe') ? 1 : 0;
    my $UnSubscribeRequestStatus = $UnSubscribe ? NEWSLETTER_ADDRESSEE_REQUEST_UNSUBSCRIBE : NEWSLETTER_ADDRESSEE_REQUEST_SUBSCRIBE;
    my $UnSubscribeStatus = $UnSubscribe ? NEWSLETTER_ADDRESSEE_UNSUBSCRIBED : NEWSLETTER_ADDRESSEE_SUBSCRIBED;

    my $SecurityCode = $Servlet->params('SecurityCode');
    $Servlet->form->executeFormError({
        'Reason'=> 'RequestManipulated',
        'Name'  => 'RequestManipulated',
    }) unless VerifySecureCode($Name.$EMail.$UnSubscribe, $SecurityCode);

    my @CampaingIDs = split(/_/, $Servlet->params('Campaigns'));
    my $aCampaigns = $Site->loadObjects( \@CampaingIDs, 'NewsletterCampaign' );

    # campaign could be deleted
    return unless scalar @$aCampaigns;

    foreach my $campaign (@$aCampaigns) {

        if($campaign->existsAddressee($EMail) ) {
            my $Addressee = $campaign->getAddressee($EMail);
            if ($Addressee->get('Unsubscribed') == $UnSubscribeRequestStatus) {
                # update existing addresse (user or not user)
                $Addressee->set({
                    'Unsubscribed'  => $UnSubscribeStatus,
                });
            }
        } else {
            $Servlet->form->executeFormError({
                'Reason'=> 'AddresseeNotFound',
                'Name'  => 'AddresseeNotFound',
            }) ;
        }
    }
    $self->SendSubscribeMail($Servlet, $Name, $EMail, $UnSubscribe, $aCampaigns) if scalar @$aCampaigns;

    $Servlet->vars('LastAction', $UnSubscribe?'Unsubscribe':'Subscribe');
    $Servlet->vars('LastActionCampaigns', [map {$_->tleHash} @$aCampaigns]);

    return;
}

sub SendSubscribeMail {
    my $self = shift;

    my ($Servlet, $Name, $EMail, $UnSubscribe, $aCampaigns) = @_;
    my $Site = $Servlet->object;
    my $User = $Servlet->user;

    # select mail type (subscribe or unsubscribe)
    my $MailType = $self->getMailType( $Site, $UnSubscribe ? 'UnsubscribeConfirmation' : 'SubscribeConfirmation');

    return unless defined $MailType && $MailType->get('IsActive');

    # create ans send mail
    my %MailParams = (
        'TO'          => $EMail,
        'HTMLAllowed' => defined $User ? $User->get('IsHtmlEMailAllowed') : 1,
    );
    my %Vars = (
        'NewsletterCampaigns' => [map { $_->tleHash } @$aCampaigns],
        'User'                => defined $User ? $User->tleHash : undef,
        'Name'                => $Name,
        'EMail'               => $EMail,
    );
    my %RegionalParams = (
        Locale      => $Servlet->params('Locale'),
        Language    => $Servlet->params('Language'),
    );
    eval {$MailType->send( $Site, \%MailParams, \%RegionalParams, \%Vars );};
    if (ExistsError()) {
        my $Error = GetError();
        LogError($Error);
        $Servlet->params('SEND_MAIL_FAILED',1);
    }


    return;
}

sub SendConfirmationRequestMail {
    my $self = shift;

    my ($Servlet, $Name, $EMail, $UnSubscribe, $aCampaigns) = @_;
    my $Site = $Servlet->object;
    my $User = $Servlet->user;

    # select mail type (subscribe or unsubscribe)
    my $MailType = $self->getMailType( $Site, $UnSubscribe ? 'UnsubscribeConfirmationRequest' : 'SubscribeConfirmationRequest');
    unless (defined $MailType) {
        $Servlet->params('SEND_MAIL_FAILED',1);
        return;
    }

    # create ans send mail
    my %MailParams = (
        'TO'          => $EMail,
        'HTMLAllowed' => defined $User ? $User->get('IsHtmlEMailAllowed') : 1,
    );
    my %Vars = (
        'NewsletterCampaigns' => [map { $_->tleHash } @$aCampaigns],
        'NewsletterCampaigns_AsString' => join('_', map {$_->id} @$aCampaigns),
        'User'                => defined $User ? $User->tleHash : undef,
        'SecurityCode'        => CreateSecureCode($Name.$EMail.$UnSubscribe),
        'Name_Encoded'        => Scramble($Name, ''),
        'EMail_Encoded'       => Scramble($EMail, ''),
        'Name'                => $Name,
        'EMail'               => $EMail,
    );
    my %RegionalParams = (
        Locale      => $Servlet->params('Locale'),
        Language    => $Servlet->params('Language'),
    );

    eval {$MailType->send( $Site, \%MailParams, \%RegionalParams, \%Vars );};

    if (ExistsError()) {
        my $Error = GetError();
        LogError($Error);

        if ( $Error->code eq 'PARAMETER_MISSING' ) {
            $Servlet->form->executeFormError({
                'Reason'=> 'SendMailFailed',
                'Form' => 'ProductRecommendation',
                'Name' => 'ReceiptEmail',
            });
        } else {
            $Servlet->params('SEND_MAIL_FAILED',1);
        }
    }

    return;
}

# get mailtype object by alias
sub getMailType {
    my $self = shift;
    my( $Site, $Alias ) = @_;

    return $Site->folder('MailTypes')->child($Alias);
}

1;
