#========================================================================================
# package      DE_EPAGES::Product::API::Object::ProductAttribute
# state        public
# base         E_EPAGES::Object::API::Object::Attribute
#----------------------------------------------------------------------------------------
# description  object interface for ProductAttribute.
#========================================================================================
package DE_EPAGES::Product::API::Object::ProductAttribute;
use base qw( DE_EPAGES::Object::API::Object::Attribute );

use strict;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Product::API::Feature;

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ProductAttribute = $Package->insert($hInfo);
#----------------------------------------------------------------------------------------
# description  insert this info as object
#----------------------------------------------------------------------------------------
# input        $hInfo | attribute values | ref.hash
# return       $ProductAttribute | new product attribute object | DE_EPAGES::Product::API::Object::ProductAttribute
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    my $Shop = $hInfo->{'Shop'} || $hInfo->{Parent}->getSite();
    $class->testFeature($hInfo,$Shop);

    my $self = $class->SUPER::insert($hInfo);

    my %Info = ( 'Shop' => $Shop );
    $class->incrementFeature(\%Info);

    return $self;
}

#========================================================================================
# function     testFeature
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->testFeature($hInfo)
#----------------------------------------------------------------------------------------
# description  Tests if feature limit has been exceeded. Called from insert()
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
#========================================================================================
sub testFeature {
    my $self = shift;
    my ($hInfo,$Shop) = @_;
    my $MaxValue;
    my $NewValue;
    my $ProductType = $hInfo->{Parent}->parent;


    # for unity shops we don't limit the count of variation attributes
    # variation attributes are counted like normal attributes
    unless ($Shop->get('IsUnityRegistered')){
        $MaxValue = $Shop->featureMaxValue( 'Variations' );
        $NewValue = 1 + scalar grep { $_->get('Type') =~ /^PreDef/ } @{$ProductType->child('Attributes')->children};
        if ( $hInfo->{'Type'} =~ /^PreDef/ && $NewValue > $MaxValue) {
            Error( 'VariationFeatureLimitExceeded', {
                Feature  => 'Variations',
                MaxValue => $MaxValue,
                NewValue => $NewValue,
                Shop => $Shop
            });
        }
    }


    $MaxValue = $Shop->featureMaxValue( 'ProductTypeAttributes' );
    $NewValue = 1 + scalar @{$ProductType->get('UserDefinedAttributes')};
    if ($NewValue > $MaxValue) {
        Error( 'FeatureLimitExceeded', {
            Feature  => 'ProductTypeAttributes',
            MaxValue => $MaxValue,
            NewValue => $NewValue,
            Shop => $Shop
        });
    }

    return;
}

#========================================================================================
# function     incrementFeature
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->incrementFeature($hInfo)
#----------------------------------------------------------------------------------------
# description  Increments ProductType feature for a shop. Called from insert()
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
#========================================================================================
sub incrementFeature {
    my $self = shift;
    my ($hInfo) = @_;
    my $Shop = $hInfo->{'Shop'};
    my $CountAttributes = DE_EPAGES::Product::API::Feature::CountFeatureProductTypeAttributes($Shop);
    $Shop->featureCurrentValue( 'ProductTypeAttributes', $CountAttributes );

    unless ($Shop->get('IsUnityRegistered')){
        my $CountVariations = DE_EPAGES::Product::API::Feature::CountFeatureVariations($Shop);
        $Shop->featureCurrentValue( 'Variations', $CountVariations );
    }

    return;
}

#========================================================================================
# function     delete
# state        private
# syntax       $Object->delete;
# description  Deletes this object, therefore the hook 'OBJ_DeleteProductType' is triggered.
#========================================================================================
sub delete {
    my $self = shift;

    $self->SUPER::delete;
    $self->decrementFeature;
    return;
}

#========================================================================================
# function     decrementFeature
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->decrementFeature
#----------------------------------------------------------------------------------------
# description  Decrements ProductType feature for a shop. Called from delete()
#----------------------------------------------------------------------------------------
#========================================================================================
sub decrementFeature {
    my $self = shift;

    # count the remaining product attributes
    my $Shop = $self->getSite();

    my $CountAttributes = DE_EPAGES::Product::API::Feature::CountFeatureProductTypeAttributes($Shop);
    $Shop->featureCurrentValue( 'ProductTypeAttributes', $CountAttributes, 1 );

    unless ($Shop->get('IsUnityRegistered')){
        my $CountVariations = DE_EPAGES::Product::API::Feature::CountFeatureVariations($Shop);
        $Shop->featureCurrentValue( 'Variations', $CountVariations, 1 );
    }

    return;
}

#========================================================================================
# function     canInvalidatePageCache
# state        public
#----------------------------------------------------------------------------------------
# syntax       $CanInvalidate = $Object->canInvalidatePageCache;
#----------------------------------------------------------------------------------------
# description  Overwrites method DE_EPAGES::Object::API::Object::Object->canInvalidatePageCache
#               to invalidate page cache.
#----------------------------------------------------------------------------------------
# return       $CanInvalidate | always true | boolean
#========================================================================================
sub canInvalidatePageCache {
    return 1;
}

1;
