#==============================================================================
# package      DE_EPAGES::TrustedRating::API::TLE::TrustedRatingHandler
# state        public
#==============================================================================
package DE_EPAGES::TrustedRating::API::TLE::TrustedRatingHandler;

use strict;
use DE_EPAGES::Core::API::Log qw ( GetLog );
use Encode;
use MIME::Base64;

#========================================================================================
# function     RegisterHandlerProc
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RegisterHandlerProc( $hParams );
#----------------------------------------------------------------------------------------
# description  this function register this attribute handle on the processor via hook
#               function on hook TLEProcessorRegistration
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments keys :
#               <ul>
#                   <li>Processor - tle processor - object
#               </ul> | ref.hash
#========================================================================================
sub RegisterHandlerProc {
    my ($Params) = @_;
    GetLog->debug( 'TrustedRatingHandler.RegisterHandlerProc' );

    __PACKAGE__->new()->register( $Params->{'Processor'} );
}

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler = new DE_EPAGES::Object::API::TLE::IncludeTemplateHandler;
# example      my $PrefixHandler  = new DE_EPAGES::Object::API::TLE::IncludeTemplateHandler;
#----------------------------------------------------------------------------------------
# description  create a new handler
#----------------------------------------------------------------------------------------
# return       $Handler | blessed object | object
#========================================================================================
sub new { bless {}, shift; }

#========================================================================================
# function     register
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->register( $Processor );
#----------------------------------------------------------------------------------------
# description  Registers this handle on the processor
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
#========================================================================================
sub register {
    my $self = shift;
    my ($Processor) = @_;
    GetLog->debug( 'TrustedRatingHandler.register' );

    $Processor->registerHandler('FunctionHandler', $self, 'ENCODESTRING');
    $Processor->registerHandler('FunctionHandler', $self, 'CANTRUSTEDRATING');
    $Processor->registerHandler('FunctionHandler', $self, 'POSSIBLETRUSTEDSHOPSLANGUAGES');

    return;
}

#========================================================================================
# function     ENCODESTRING
# state        public
#----------------------------------------------------------------------------------------
# syntax       #ENCODESTRING(#String)
# example      #ENCODESTRING(#Email)
#----------------------------------------------------------------------------------------
# description  Returns the base64 encoded value of a string.
#----------------------------------------------------------------------------------------
# input        #Name | string | string
# return       #ReturnValue | encoded value of string | ref.string
#========================================================================================
sub ENCODESTRING {
    my $self = shift;
    my ($Processor, $aParams) = @_;

    my ($Name) = @$aParams;
    return encode_base64( encode_utf8 $Name);
}

#========================================================================================
# function     POSSIBLETRUSTEDSHOPSLANGUAGES
# state        public
#----------------------------------------------------------------------------------------
# syntax       POSSIBLETRUSTEDSHOPSLANGUAGES(#SHOP)
# example      POSSIBLETRUSTEDSHOPSLANGUAGES(#SHOP)
#----------------------------------------------------------------------------------------
# description  Takes all Shop languages returns this without this Languages which are used 
#               for other Trusted Shops Certificates
#----------------------------------------------------------------------------------------
# input        $TsID | TrustedShopID | string
# return       $ReturnValue | ar  | boolean
# return       $ReturnValue | Array with possible languages| refarray
#========================================================================================
sub POSSIBLETRUSTEDSHOPSLANGUAGES {
    my $self = shift;
    my ($Processor, $aParams) = @_;

    my ( $Shop ) = @$aParams;
    my $aTrustedShopsCertificates = $Shop->{'TrustedShopsCertificates'};
    my $aLanguages = $Shop->{'Languages'};
    my @PossibleTrustedShopLanguages ;
    foreach my $Language (@$aLanguages){
        my $IsLanguageSettet = 0;
        foreach my $TrustedShopsCertificate (@$aTrustedShopsCertificates){
            if ($TrustedShopsCertificate->{'CertificationLanguage'} eq $Language->{'Code2'}){
                $IsLanguageSettet = 1;
            }
        }
        unless ( $IsLanguageSettet ){
            push( @PossibleTrustedShopLanguages,
                {
                'LocalID'  => $Language->{'LanguageID'} ,
                'Native' => $Language->{'Native'},
                }
            );
        }
    }
    return \@PossibleTrustedShopLanguages;
}

1;