#==============================================================================
# package      DE_EPAGES::TrustedShops::API::RemoteTrustedShops
# state        private
#==============================================================================
package DE_EPAGES::TrustedShops::API::RemoteTrustedShops;
use base qw (Exporter);

use strict;
our @EXPORT_OK = qw (
    CheckCertificate
    CheckLogin
    GetProtectionItems
    GetRequestState
    RequestForProtection
);


use SOAP::Lite;# +trace;
use DE_EPAGES::Core::API::Error qw (ExistsError GetError);
use DE_EPAGES::Core::API::Log qw (LogError);
use DE_EPAGES::Core::API::TestParameter qw (TestObject TestString);
use DE_EPAGES::Object::API::Factory qw (LoadRootObject);
use DE_EPAGES::TrustedShops::API::Constants qw (TS_ID_LENGTH
                                                %WEBSERVICE_FE_TEST
                                                %WEBSERVICE_BE_TEST
                                                %WEBSERVICE_FE_LIVE
                                                %WEBSERVICE_BE_LIVE);
use DE_EPAGES::TrustedShops::API::Tools qw (LogTrustedShops);
use DE_EPAGES::ThirdPartyConfig::API::ThirdPartyConfig qw (GetDefaultTimeOut);

#========================================================================================
# function     CheckCertificate
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $hResult = CheckCertificate('X0C8EABCD008334175EC5F12FE969D097');
#               my $hResult = CheckCertificate('X0C8EABCD008334175EC5F12FE969D097', 1);
#----------------------------------------------------------------------------------------
# description  call webservice function checkCertificate
#----------------------------------------------------------------------------------------
# input        $tsId | trusted shops id / certificate alias | string
# input        $Shop | the Shop object | ref_hash
# return       $hResult | webservice result (undef on failure); hash with keys:
#               <ul>
#                 <li>certificationLanguage - two char language code - string</li>
#                 <li>stateEnum - certificate status (PRODUCTION, TEST, INVALID_TS_ID, CANCELLED, DISABLED, INTEGRATION) - string</li>
#                 <li>tsID - trusted shops id / certificate alias - string</li>
#                 <li>typeEnum - certificate type (EXCELLENCE, CLASSIC) - string</li>
#                 <li>url - shop url - string</li>
#               </ul>
#               | ref.hash
#========================================================================================
sub CheckCertificate {
    my ($TsID, $Shop) = @_;
    return undef unless defined $Shop;
    my $IsInTestMode = $Shop->get('IsInternalTestShop');
    TestString('TsID', $TsID, TS_ID_LENGTH, TS_ID_LENGTH);

    my %WebServiceData = $IsInTestMode  ?  %WEBSERVICE_BE_TEST : %WEBSERVICE_BE_LIVE;
    my $client = SOAP::Lite
                    ->uri($WebServiceData{'uri'})
                    ->proxy($WebServiceData{'proxy'}, 'timeout' => GetDefaultTimeOut)
                    ->ns($WebServiceData{'namespace'});

    LogTrustedShops('CheckCertificate params', {'tsId' => $TsID}, 'CheckCertificate');
    my $result = undef;
    eval {
        $result = $client->checkCertificate(SOAP::Data->name('tsId' => $TsID));
    };
    if (ExistsError()) {
        LogError(GetError());
    }
    if (!$result || $result->fault) {
        LogTrustedShops('CheckCertificate failed', {'tsId' => $TsID}, 'CheckCertificate');
        return undef;
    }
    my $hResult = $result->result;
    LogTrustedShops("CheckCertificate result for tsId $TsID", $hResult, 'CheckCertificate');
    return $hResult;
}

#========================================================================================
# function     _ResultItemsToArray
# state        _private
#----------------------------------------------------------------------------------------
# syntax       _ResultItemsToArray( $hResult );
#----------------------------------------------------------------------------------------
# input        $hResult | reference to hash or array | ref
#----------------------------------------------------------------------------------------
# description  If $hResult->{item} is a Hash_ref it will encapsulated in an array
#----------------------------------------------------------------------------------------
# return       $ahItems | The item Element as an array ref | ref.array.hash
#========================================================================================
sub _ResultItemsToArray {
    my ($hResult) = @_;
    if ( ref( $hResult->{'item'} ) ne 'ARRAY' ) {
        # xml simple returns only a hash reference if there is only one element
        # but the return value have to be a array reference
        return [ $hResult->{'item'} ];
    }
    else {
        return $hResult->{'item'};
    }
}

#========================================================================================
# function     GetProtectionItems
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $ahResult = GetProtectionItems('X0C8EABCD008334175EC5F12FE969D097');
#               my $ahResult = GetProtectionItems('X0C8EABCD008334175EC5F12FE969D097', 1);
#----------------------------------------------------------------------------------------
# description  call webservice function getProtectionItems
#----------------------------------------------------------------------------------------
# input        $tsId | trusted shops id / certificate alias | string
# input        $IsInTestMode | if true use test webservice | boolean
# return       $ahResult | webservice result (undef on failure); list of hashes with keys:
#               <ul>
#                 <li>creationDate - creation date (e.g. 2008-07-01T14:00:00+02:00) - string</li>
#                 <li>currency - 3 char currency code (e.g. EUR) - string</li>
#                 <li>grossFee - gross price - float</li>
#                 <li>id - item id - integer</li>
#                 <li>netFee - net price - float</li>
#                 <li>protectedAmountDecimal - max amount protected by this item - float</li>
#                 <li>tsProductID - product id - string</li>
#               </ul> | ref.array.hash
#========================================================================================
sub GetProtectionItems {
    my ($TsID, $IsInTestMode) = @_;
    TestString('TsID', $TsID, TS_ID_LENGTH, TS_ID_LENGTH);

    my %WebServiceData = $IsInTestMode  ?  %WEBSERVICE_BE_TEST : %WEBSERVICE_BE_LIVE;
    my $client = SOAP::Lite
                    ->uri($WebServiceData{'uri'})
                    ->proxy($WebServiceData{'proxy'}, 'timeout' => GetDefaultTimeOut)
                    ->ns($WebServiceData{'namespace'});

    LogTrustedShops('GetProtectionItems params', {'tsId' => $TsID}, 'GetProtectionItems');
    my $result = undef;
    eval {
        $result = $client->getProtectionItems(SOAP::Data->name('tsId' => $TsID));
    };
    if (ExistsError()) {
        LogError(GetError());
    }
    if (!$result || $result->fault) {
        LogTrustedShops('GetProtectionItems failed', {'tsId' => $TsID}, 'GetProtectionItems');
        return undef;
    }
    my $hResult = $result->result;
    LogTrustedShops("GetProtectionItems result for tsId $TsID", $hResult, 'GetProtectionItems');
    return _ResultItemsToArray($hResult);
}

#========================================================================================
# function     CheckLogin
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $result = CheckLogin('X0C8EABCD008334175EC5F12FE969D097', $wslogin, $wspassword);
#----------------------------------------------------------------------------------------
# description  call webservice function checkLogin
#----------------------------------------------------------------------------------------
# input        $tsId | trusted shops id / certificate alias | string
# input        $wsUser | webservice user name | string
# input        $wsPassword | webservice password | string
# input        $IsInTestMode | if true use test webservice | boolean
# return       $result | return value from checkLogin (1 - success) or undef on error | integer
#========================================================================================
sub CheckLogin {
    my ($TsID, $WsUser, $WsPassword, $IsInTestMode) = @_;
    TestString('TsID', $TsID, TS_ID_LENGTH, TS_ID_LENGTH);
    TestString('WsUser', $WsUser);
    TestString('WsPassword', $WsPassword);

    my %WebServiceData = $IsInTestMode  ?  %WEBSERVICE_BE_TEST : %WEBSERVICE_BE_LIVE;
    my $client = SOAP::Lite
                    ->uri($WebServiceData{'uri'})
                    ->proxy($WebServiceData{'proxy'}, 'timeout' => GetDefaultTimeOut)
                    ->ns($WebServiceData{'namespace'});

    LogTrustedShops('CheckLogin params', {'tsId' => $TsID, 'wsUser' => $WsUser, 'wsPassword' => $WsPassword}, 'CheckLogin');
    my $result = undef;
    eval {
        $result = $client->checkLogin(
            SOAP::Data->name('tsId'       => $TsID),
            SOAP::Data->name('wsUser'     => $WsUser),
            SOAP::Data->name('wsPassword' => $WsPassword)
        );
    };
    if (ExistsError()) {
        LogError(GetError());
    }
    if (!$result || $result->fault) {
        LogTrustedShops('CheckLogin failed', {'tsId' => $TsID, 'wsUser' => $WsUser, 'wsPassword' => $WsPassword}, 'CheckLogin');
        return undef;
    }
    my $ret = $result->result;
    LogTrustedShops("CheckLogin result for tsId $TsID", $ret, 'CheckLogin');
    return $ret;
}

#========================================================================================
# function     RequestForProtection
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $result = RequestForProtection($Order);
#----------------------------------------------------------------------------------------
# description  call webservice function requestForProtection
#----------------------------------------------------------------------------------------
# input        $Order | order to send the webservice request for | object
# return       $result | webservice result:
#               <ul>
#                  <li>undef - failure</li>
#                  <li>negative value - declined</li>
#                  <li>positive value - success, value is the application id</li>
#               </ul> | integer
#========================================================================================
sub RequestForProtection {
    my ($Order) = @_;
    TestObject('Order', $Order, 'DE_EPAGES::Order::API::Object::CustomerOrder');

    my $Container = $Order->container;
    return undef unless defined $Container;
    return undef unless $Container->existsChild('LineItemTSProtectionItem');

    my $ProtectionItem = $Container->child('LineItemTSProtectionItem')->get('ProtectionItem');
    return undef unless defined $ProtectionItem;

    my $Certificate = $ProtectionItem->get('TrustedShopsCertificate');
    return undef unless defined $Certificate;

    # get all data necessary for webservice request
    my $tsId              = $Certificate->alias;
    my $tsProductId       = $ProtectionItem->alias;
    my $amount            = $Container->get('GrandTotal'); # includes the amount of the ProtectionItem, but TS say that is the right way of doing it
    my $currency          = $Container->get('CurrencyID');
    my $paymentType       = $Container->get('TrustedShopsPaymentMethodExcellent');
    my $buyerEmail        = $Order->get('BillingAddress')->get('EMail');
    my $shopCustomerID    = $Order->get('Customer')->alias;
    my $shopOrderID       = $Order->alias;
    my $orderDate         = $Order->get('CreationDate');
    my $shopSystemVersion = 'ePages ' . LoadRootObject()->get('EpagesVersion');
    my $wsUser            = $Certificate->get('WebserviceUsername');
    my $wsPassword        = $Certificate->get('WebservicePassword');

    # debugging, request etc.
    my $IsInTestMode = $Order->getSite()->get('IsInternalTestShop');
    my %WebServiceData = $IsInTestMode  ?  %WEBSERVICE_FE_TEST : %WEBSERVICE_FE_LIVE;
    my $client = SOAP::Lite
                    ->uri($WebServiceData{'uri'})
                    ->proxy($WebServiceData{'proxy'}, 'timeout' => GetDefaultTimeOut)
                    ->ns($WebServiceData{'namespace'});

    LogTrustedShops('RequestForProtection params for order ' . $Order->id . ' (' . $Order->pathString .')', {
        'tsId'              => $tsId,
        'tsProductId'       => $tsProductId,
        'amount'            => $amount,
        'currency'          => $currency,
        'paymentType'       => $paymentType,
        'buyerEmail'        => $buyerEmail,
        'shopCustomerID'    => $shopCustomerID,
        'shopOrderID'       => $shopOrderID,
        'orderDate'         => $orderDate->iso8601,
        'shopSystemVersion' => $shopSystemVersion,
        'wsUser'            => $wsUser,
        'wsPassword'        => $wsPassword
    }, 'RequestForProtection');
    my $result = undef;
    eval {
        $result = $client->requestForProtectionV2(
            SOAP::Data->name('tsId'              => $tsId)->type('string'),
            SOAP::Data->name('tsProductId'       => $tsProductId)->type('string'),
            SOAP::Data->name('amount'            => $amount)->type('decimal'),
            SOAP::Data->name('currency'          => $currency)->type('string'),
            SOAP::Data->name('paymentType'       => $paymentType)->type('string'),
            SOAP::Data->name('buyerEmail'        => $buyerEmail)->type('string'),
            SOAP::Data->name('shopCustomerID'    => $shopCustomerID)->type('string'),
            SOAP::Data->name('shopOrderID'       => $shopOrderID)->type('string'),
            SOAP::Data->name('orderDate')->type('dateTime' => $orderDate),
            SOAP::Data->name('shopSystemVersion' => $shopSystemVersion)->type('string'),
            SOAP::Data->name('wsUser'            => $wsUser)->type('string'),
            SOAP::Data->name('wsPassword'        => $wsPassword)->type('string')
        );
    };
    if (ExistsError()) {
        LogError(GetError());
    }
    if (!$result || $result->fault) {
        LogTrustedShops('RequestForProtection failed for order ' . $Order->id . ' (' . $Order->pathString .')', {
            'tsId'        => $tsId,
            'tsProductId' => $tsProductId
        }, 'RequestForProtection');
        return undef;
    }
    my $ret = $result->result;
    LogTrustedShops("RequestForProtection result for tsProductId $tsProductId", $ret, 'RequestForProtection');
    return $ret;
}

#========================================================================================
# function     GetRequestState
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $result = GetRequestState($LineItemTSProtectionItem);
#----------------------------------------------------------------------------------------
# description  call webservice function getRequestState
#----------------------------------------------------------------------------------------
# input        $LineItemTSProtectionItem | lineitem | object
# return       $result | webservice result:
#               <ul>
#                  <li>undef - failure</li>
#                  <li>0 - not processed yet</li>
#                  <li>negative value - declined</li>
#                  <li>positive value - success, value is the guarantee no</li>
#               </ul> | integer
#========================================================================================
sub GetRequestState {
    my ($LineItemTSProtectionItem) = @_;

    my $applicationId = $LineItemTSProtectionItem->get('ApplicationID');
    return undef if(!defined($applicationId) || $applicationId <= 0);

    my $tsId = $LineItemTSProtectionItem->get('TrustedShopsID');
    return undef unless defined $tsId;

    # debugging, request etc.
    my $IsInTestMode = $LineItemTSProtectionItem->getSite()->get('IsInternalTestShop');
    my %WebServiceData = $IsInTestMode  ?  %WEBSERVICE_FE_TEST : %WEBSERVICE_FE_LIVE;
    my $client = SOAP::Lite
                    ->uri($WebServiceData{'uri'})
                    ->proxy($WebServiceData{'proxy'}, 'timeout' => GetDefaultTimeOut)
                    ->ns($WebServiceData{'namespace'});

    LogTrustedShops('GetRequestState params for line item ' . $LineItemTSProtectionItem->id . ' (' . $LineItemTSProtectionItem->pathString .')', {
        'tsId'          => $tsId,
        'applicationId' => $applicationId
    }, 'GetRequestState');
    my $result = undef;
    eval {
        $result = $client->getRequestState(
            SOAP::Data->name('tsId'           => $tsId),
            SOAP::Data->name('applicationId'  => $applicationId)
        );
    };
    if (ExistsError()) {
        LogError(GetError());
    }
    if (!$result || $result->fault) {
        LogTrustedShops('GetRequestState failed for line item ' . $LineItemTSProtectionItem->id . ' (' . $LineItemTSProtectionItem->pathString .')', {
            'tsId'          => $tsId,
            'applicationId' => $applicationId
        }, 'GetRequestState');
        return undef;
    }
    my $ret = $result->result;
    LogTrustedShops('GetRequestState result for line item ' . $LineItemTSProtectionItem->id . ' (' . $LineItemTSProtectionItem->pathString .')', $ret, 'GetRequestState');
    return $ret;
}

1;
